diff --git a/briar-android-tests/src/test/java/org/briarproject/IntroductionIntegrationTest.java b/briar-android-tests/src/test/java/org/briarproject/IntroductionIntegrationTest.java
index cb219c32294887759480905ead781be56a38073a..f049c142a86f0b4a1ac192ec10525432d51e2e46 100644
--- a/briar-android-tests/src/test/java/org/briarproject/IntroductionIntegrationTest.java
+++ b/briar-android-tests/src/test/java/org/briarproject/IntroductionIntegrationTest.java
@@ -42,6 +42,7 @@ import org.briarproject.introduction.MessageSender;
 import org.briarproject.lifecycle.LifecycleModule;
 import org.briarproject.properties.PropertiesModule;
 import org.briarproject.sync.SyncModule;
+import org.briarproject.system.SystemModule;
 import org.briarproject.transport.TransportModule;
 import org.junit.After;
 import org.junit.Before;
@@ -944,6 +945,7 @@ public class IntroductionIntegrationTest extends BriarTestCase {
 			this.accept = accept;
 		}
 
+		@Override
 		public void eventOccurred(Event e) {
 			if (e instanceof MessageValidatedEvent) {
 				MessageValidatedEvent event = (MessageValidatedEvent) e;
@@ -1010,6 +1012,7 @@ public class IntroductionIntegrationTest extends BriarTestCase {
 		public volatile boolean response2Received = false;
 		public volatile boolean aborted = false;
 
+		@Override
 		public void eventOccurred(Event e) {
 			if (e instanceof MessageValidatedEvent) {
 				MessageValidatedEvent event = (MessageValidatedEvent) e;
@@ -1050,7 +1053,7 @@ public class IntroductionIntegrationTest extends BriarTestCase {
 		component.inject(new ContactModule.EagerSingletons());
 		component.inject(new TransportModule.EagerSingletons());
 		component.inject(new SyncModule.EagerSingletons());
+		component.inject(new SystemModule.EagerSingletons());
 		component.inject(new PropertiesModule.EagerSingletons());
 	}
-
 }
diff --git a/briar-android-tests/src/test/java/org/briarproject/IntroductionIntegrationTestComponent.java b/briar-android-tests/src/test/java/org/briarproject/IntroductionIntegrationTestComponent.java
index 1a085329148d42f1fa5e118622d93999c8fcb405..733082ef0bd446a8c9ab805e6a11091de9653833 100644
--- a/briar-android-tests/src/test/java/org/briarproject/IntroductionIntegrationTestComponent.java
+++ b/briar-android-tests/src/test/java/org/briarproject/IntroductionIntegrationTestComponent.java
@@ -21,6 +21,7 @@ import org.briarproject.introduction.MessageSender;
 import org.briarproject.lifecycle.LifecycleModule;
 import org.briarproject.properties.PropertiesModule;
 import org.briarproject.sync.SyncModule;
+import org.briarproject.system.SystemModule;
 import org.briarproject.transport.TransportModule;
 
 import javax.inject.Singleton;
@@ -29,9 +30,9 @@ import dagger.Component;
 
 @Singleton
 @Component(modules = {
-		TestSystemModule.class,
 		TestDatabaseModule.class,
 		TestPluginsModule.class,
+		TestSeedProviderModule.class,
 		LifecycleModule.class,
 		IntroductionModule.class,
 		DatabaseModule.class,
@@ -42,6 +43,7 @@ import dagger.Component;
 		TransportModule.class,
 		ClientsModule.class,
 		SyncModule.class,
+		SystemModule.class,
 		DataModule.class,
 		PropertiesModule.class
 })
@@ -61,6 +63,8 @@ public interface IntroductionIntegrationTestComponent {
 
 	void inject(SyncModule.EagerSingletons init);
 
+	void inject(SystemModule.EagerSingletons init);
+
 	void inject(TransportModule.EagerSingletons init);
 
 	LifecycleManager getLifecycleManager();
@@ -84,5 +88,4 @@ public interface IntroductionIntegrationTestComponent {
 	MessageSender getMessageSender();
 
 	IntroductionGroupFactory getIntroductionGroupFactory();
-
 }
diff --git a/briar-android-tests/src/test/java/org/briarproject/MessageSizeIntegrationTest.java b/briar-android-tests/src/test/java/org/briarproject/MessageSizeIntegrationTest.java
index 01157a3e8079c92b886f49d7abd83e01383b73ed..5e5b877cc86364d8fefdffcefac2c0fd6647426d 100644
--- a/briar-android-tests/src/test/java/org/briarproject/MessageSizeIntegrationTest.java
+++ b/briar-android-tests/src/test/java/org/briarproject/MessageSizeIntegrationTest.java
@@ -13,6 +13,7 @@ import org.briarproject.api.messaging.PrivateMessage;
 import org.briarproject.api.messaging.PrivateMessageFactory;
 import org.briarproject.api.sync.GroupId;
 import org.briarproject.api.sync.MessageId;
+import org.briarproject.system.SystemModule;
 import org.junit.Test;
 
 import javax.inject.Inject;
@@ -39,6 +40,7 @@ public class MessageSizeIntegrationTest extends BriarTestCase {
 		MessageSizeIntegrationTestComponent component =
 				DaggerMessageSizeIntegrationTestComponent.builder().build();
 		component.inject(this);
+		component.inject(new SystemModule.EagerSingletons());
 	}
 
 	@Test
diff --git a/briar-android-tests/src/test/java/org/briarproject/MessageSizeIntegrationTestComponent.java b/briar-android-tests/src/test/java/org/briarproject/MessageSizeIntegrationTestComponent.java
index 9ec86f7c7f6976debb7e07b992714a888fe9f7d0..fe530464065456a4d633f83621d6a2bacf40eb87 100644
--- a/briar-android-tests/src/test/java/org/briarproject/MessageSizeIntegrationTestComponent.java
+++ b/briar-android-tests/src/test/java/org/briarproject/MessageSizeIntegrationTestComponent.java
@@ -9,6 +9,7 @@ import org.briarproject.forum.ForumModule;
 import org.briarproject.identity.IdentityModule;
 import org.briarproject.messaging.MessagingModule;
 import org.briarproject.sync.SyncModule;
+import org.briarproject.system.SystemModule;
 
 import javax.inject.Singleton;
 
@@ -18,7 +19,7 @@ import dagger.Component;
 @Component(modules = {
 		TestDatabaseModule.class,
 		TestLifecycleModule.class,
-		TestSystemModule.class,
+		TestSeedProviderModule.class,
 		ClientsModule.class,
 		CryptoModule.class,
 		DataModule.class,
@@ -27,8 +28,12 @@ import dagger.Component;
 		ForumModule.class,
 		IdentityModule.class,
 		MessagingModule.class,
-		SyncModule.class
+		SyncModule.class,
+		SystemModule.class
 })
 public interface MessageSizeIntegrationTestComponent {
+
 	void inject(MessageSizeIntegrationTest testCase);
+
+	void inject(SystemModule.EagerSingletons init);
 }
diff --git a/briar-android-tests/src/test/java/org/briarproject/SimplexMessagingIntegrationTest.java b/briar-android-tests/src/test/java/org/briarproject/SimplexMessagingIntegrationTest.java
index 8a2d43e2c490483d1c7053ffb610b5241b60a97f..3cbf69a6da1e263fa83db6feabc4aa03ae3448b7 100644
--- a/briar-android-tests/src/test/java/org/briarproject/SimplexMessagingIntegrationTest.java
+++ b/briar-android-tests/src/test/java/org/briarproject/SimplexMessagingIntegrationTest.java
@@ -21,6 +21,7 @@ import org.briarproject.api.transport.KeyManager;
 import org.briarproject.api.transport.StreamContext;
 import org.briarproject.api.transport.StreamReaderFactory;
 import org.briarproject.api.transport.StreamWriterFactory;
+import org.briarproject.system.SystemModule;
 import org.junit.After;
 import org.junit.Before;
 import org.junit.Test;
@@ -57,8 +58,10 @@ public class SimplexMessagingIntegrationTest extends BriarTestCase {
 		assertTrue(testDir.mkdirs());
 		alice = DaggerSimplexMessagingIntegrationTestComponent.builder()
 				.testDatabaseModule(new TestDatabaseModule(aliceDir)).build();
+		alice.inject(new SystemModule.EagerSingletons());
 		bob = DaggerSimplexMessagingIntegrationTestComponent.builder()
 				.testDatabaseModule(new TestDatabaseModule(bobDir)).build();
+		bob.inject(new SystemModule.EagerSingletons());
 	}
 
 	@Test
@@ -183,6 +186,7 @@ public class SimplexMessagingIntegrationTest extends BriarTestCase {
 
 		private volatile boolean messageAdded = false;
 
+		@Override
 		public void eventOccurred(Event e) {
 			if (e instanceof MessageAddedEvent) messageAdded = true;
 		}
diff --git a/briar-android-tests/src/test/java/org/briarproject/SimplexMessagingIntegrationTestComponent.java b/briar-android-tests/src/test/java/org/briarproject/SimplexMessagingIntegrationTestComponent.java
index 28480fb1a13c1a0ff44b70cf5d436b33f66a80d3..9de8e9c5ea755fac01073114a5281c0d6a2470b5 100644
--- a/briar-android-tests/src/test/java/org/briarproject/SimplexMessagingIntegrationTestComponent.java
+++ b/briar-android-tests/src/test/java/org/briarproject/SimplexMessagingIntegrationTestComponent.java
@@ -21,6 +21,7 @@ import org.briarproject.lifecycle.LifecycleModule;
 import org.briarproject.messaging.MessagingModule;
 import org.briarproject.plugins.PluginsModule;
 import org.briarproject.sync.SyncModule;
+import org.briarproject.system.SystemModule;
 import org.briarproject.transport.TransportModule;
 
 import javax.inject.Singleton;
@@ -31,7 +32,7 @@ import dagger.Component;
 @Component(modules = {
 		TestDatabaseModule.class,
 		TestPluginsModule.class,
-		TestSystemModule.class,
+		TestSeedProviderModule.class,
 		ClientsModule.class,
 		ContactModule.class,
 		CryptoModule.class,
@@ -43,11 +44,12 @@ import dagger.Component;
 		MessagingModule.class,
 		PluginsModule.class,
 		SyncModule.class,
+		SystemModule.class,
 		TransportModule.class
 })
 public interface SimplexMessagingIntegrationTestComponent {
 
-	void inject(SimplexMessagingIntegrationTest testCase);
+	void inject(SystemModule.EagerSingletons init);
 
 	LifecycleManager getLifecycleManager();
 
diff --git a/briar-android-tests/src/test/java/org/briarproject/SyncIntegrationTestComponent.java b/briar-android-tests/src/test/java/org/briarproject/SyncIntegrationTestComponent.java
index b5d55b93574c37f24d11e6a6cb5743adae28a1ca..69710bd5a6b710e1cc94981cee256944fb845bc3 100644
--- a/briar-android-tests/src/test/java/org/briarproject/SyncIntegrationTestComponent.java
+++ b/briar-android-tests/src/test/java/org/briarproject/SyncIntegrationTestComponent.java
@@ -10,7 +10,7 @@ import dagger.Component;
 
 @Singleton
 @Component(modules = {
-		TestSystemModule.class,
+		TestSeedProviderModule.class,
 		CryptoModule.class,
 		SyncModule.class,
 		TransportModule.class
diff --git a/briar-android/src/org/briarproject/system/AndroidSystemModule.java b/briar-android/src/org/briarproject/system/AndroidSystemModule.java
index f6fb19f16d9ea7b73dec8bee6fad856c96c22100..a577ac7c3e1c4d787f80b22d44a406bcb4b170a1 100644
--- a/briar-android/src/org/briarproject/system/AndroidSystemModule.java
+++ b/briar-android/src/org/briarproject/system/AndroidSystemModule.java
@@ -15,6 +15,7 @@ import dagger.Provides;
 public class AndroidSystemModule {
 
 	@Provides
+	@Singleton
 	public SeedProvider provideSeedProvider(Application app) {
 		return new AndroidSeedProvider(app);
 	}
diff --git a/briar-api/src/org/briarproject/api/system/Timer.java b/briar-api/src/org/briarproject/api/system/Timer.java
deleted file mode 100644
index 9eb08e88b1c237fabbe97516ca30b15ef73e4fca..0000000000000000000000000000000000000000
--- a/briar-api/src/org/briarproject/api/system/Timer.java
+++ /dev/null
@@ -1,27 +0,0 @@
-package org.briarproject.api.system;
-
-import java.util.TimerTask;
-
-/**
- * A wrapper around a {@link java.util.Timer} that allows it to be replaced for
- * testing.
- */
-public interface Timer {
-
-	/** @see {@link java.util.Timer#cancel()} */
-	void cancel();
-
-	/** @see {@link java.util.Timer#purge()} */
-	int purge();
-
-	/** @see {@link java.util.Timer#schedule(TimerTask, long)} */
-	void schedule(TimerTask task, long delay);
-
-	/** @see {@link java.util.Timer#schedule(TimerTask, long, long)} */
-	void schedule(TimerTask task, long delay, long period);
-
-	/**
-	 * @see {@link java.util.Timer#scheduleAtFixedRate(TimerTask, long, long)}
-	 */
-	void scheduleAtFixedRate(TimerTask task, long delay, long period);
-}
diff --git a/briar-core/src/org/briarproject/CoreEagerSingletons.java b/briar-core/src/org/briarproject/CoreEagerSingletons.java
index 0823bccbf24c07f0daedc8b77650a332dc4fb7a1..174ee9c570b572e77861cff17229a19f0f9d8a71 100644
--- a/briar-core/src/org/briarproject/CoreEagerSingletons.java
+++ b/briar-core/src/org/briarproject/CoreEagerSingletons.java
@@ -10,6 +10,7 @@ import org.briarproject.messaging.MessagingModule;
 import org.briarproject.plugins.PluginsModule;
 import org.briarproject.properties.PropertiesModule;
 import org.briarproject.sync.SyncModule;
+import org.briarproject.system.SystemModule;
 import org.briarproject.transport.TransportModule;
 
 public interface CoreEagerSingletons {
@@ -34,5 +35,7 @@ public interface CoreEagerSingletons {
 
 	void inject(SyncModule.EagerSingletons init);
 
+	void inject(SystemModule.EagerSingletons init);
+
 	void inject(TransportModule.EagerSingletons init);
 }
diff --git a/briar-core/src/org/briarproject/CoreModule.java b/briar-core/src/org/briarproject/CoreModule.java
index 62828dfffb6a351b752e18ace8581f6e6bcdf0ab..238c859e7fa8f1d79b00613c14e92938d7ace480 100644
--- a/briar-core/src/org/briarproject/CoreModule.java
+++ b/briar-core/src/org/briarproject/CoreModule.java
@@ -61,6 +61,7 @@ public class CoreModule {
 		c.inject(new PluginsModule.EagerSingletons());
 		c.inject(new PropertiesModule.EagerSingletons());
 		c.inject(new SyncModule.EagerSingletons());
+		c.inject(new SystemModule.EagerSingletons());
 		c.inject(new TransportModule.EagerSingletons());
 		c.inject(new IntroductionModule.EagerSingletons());
 	}
diff --git a/briar-core/src/org/briarproject/lifecycle/LifecycleManagerImpl.java b/briar-core/src/org/briarproject/lifecycle/LifecycleManagerImpl.java
index 0e7810eb56904b37c93f631af2791e0f01097af0..b227943116e713cd5a277e7ad2699583b3149a7d 100644
--- a/briar-core/src/org/briarproject/lifecycle/LifecycleManagerImpl.java
+++ b/briar-core/src/org/briarproject/lifecycle/LifecycleManagerImpl.java
@@ -50,24 +50,27 @@ class LifecycleManagerImpl implements LifecycleManager {
 		executors = new CopyOnWriteArrayList<ExecutorService>();
 	}
 
+	@Override
 	public void registerService(Service s) {
 		if (LOG.isLoggable(INFO))
 			LOG.info("Registering service " + s.getClass().getName());
 		services.add(s);
 	}
 
+	@Override
 	public void registerClient(Client c) {
 		if (LOG.isLoggable(INFO))
 			LOG.info("Registering client " + c.getClass().getName());
 		clients.add(c);
 	}
 
+	@Override
 	public void registerForShutdown(ExecutorService e) {
-		if (LOG.isLoggable(INFO))
-			LOG.info("Registering executor " + e.getClass().getName());
+		LOG.info("Registering executor");
 		executors.add(e);
 	}
 
+	@Override
 	public StartResult startServices() {
 		if (!startStopSemaphore.tryAcquire()) {
 			LOG.info("Already starting or stopping");
@@ -91,7 +94,7 @@ class LifecycleManagerImpl implements LifecycleManager {
 					c.createLocalState(txn);
 					duration = System.currentTimeMillis() - start;
 					if (LOG.isLoggable(INFO)) {
-						LOG.info("Starting " + c.getClass().getName()
+						LOG.info("Starting client " + c.getClass().getName()
 								+ " took " + duration + " ms");
 					}
 				}
@@ -104,7 +107,7 @@ class LifecycleManagerImpl implements LifecycleManager {
 				s.startService();
 				duration = System.currentTimeMillis() - start;
 				if (LOG.isLoggable(INFO)) {
-					LOG.info("Starting " + s.getClass().getName()
+					LOG.info("Starting service " + s.getClass().getName()
 							+ " took " + duration + " ms");
 				}
 			}
@@ -121,6 +124,7 @@ class LifecycleManagerImpl implements LifecycleManager {
 		}
 	}
 
+	@Override
 	public void stopServices() {
 		try {
 			startStopSemaphore.acquire();
@@ -132,15 +136,22 @@ class LifecycleManagerImpl implements LifecycleManager {
 			LOG.info("Stopping services");
 			eventBus.broadcast(new ShutdownEvent());
 			for (Service s : services) {
+				long start = System.currentTimeMillis();
 				s.stopService();
-				if (LOG.isLoggable(INFO))
-					LOG.info("Service stopped: " + s.getClass().getName());
+				long duration = System.currentTimeMillis() - start;
+				if (LOG.isLoggable(INFO)) {
+					LOG.info("Stopping service " + s.getClass().getName()
+							+ " took " + duration + " ms");
+				}
 			}
 			for (ExecutorService e : executors) e.shutdownNow();
 			if (LOG.isLoggable(INFO))
 				LOG.info(executors.size() + " executors shut down");
+			long start = System.currentTimeMillis();
 			db.close();
-			LOG.info("Database closed");
+			long duration = System.currentTimeMillis() - start;
+			if (LOG.isLoggable(INFO))
+				LOG.info("Closing database took " + duration + " ms");
 			shutdownLatch.countDown();
 		} catch (DbException e) {
 			if (LOG.isLoggable(WARNING)) LOG.log(WARNING, e.toString(), e);
@@ -151,14 +162,17 @@ class LifecycleManagerImpl implements LifecycleManager {
 		}
 	}
 
+	@Override
 	public void waitForDatabase() throws InterruptedException {
 		dbLatch.await();
 	}
 
+	@Override
 	public void waitForStartup() throws InterruptedException {
 		startupLatch.await();
 	}
 
+	@Override
 	public void waitForShutdown() throws InterruptedException {
 		shutdownLatch.await();
 	}
diff --git a/briar-core/src/org/briarproject/plugins/PluginManagerImpl.java b/briar-core/src/org/briarproject/plugins/PluginManagerImpl.java
index 78c3cd707006a1b9f2afb1d5eba4f689892e09f6..b93756b18bac6bb411d71afe1730fa3847857e71 100644
--- a/briar-core/src/org/briarproject/plugins/PluginManagerImpl.java
+++ b/briar-core/src/org/briarproject/plugins/PluginManagerImpl.java
@@ -3,18 +3,13 @@ package org.briarproject.plugins;
 import org.briarproject.api.TransportId;
 import org.briarproject.api.contact.ContactId;
 import org.briarproject.api.db.DbException;
-import org.briarproject.api.event.ConnectionClosedEvent;
-import org.briarproject.api.event.ContactStatusChangedEvent;
-import org.briarproject.api.event.Event;
 import org.briarproject.api.event.EventBus;
-import org.briarproject.api.event.EventListener;
 import org.briarproject.api.event.TransportDisabledEvent;
 import org.briarproject.api.event.TransportEnabledEvent;
 import org.briarproject.api.lifecycle.IoExecutor;
 import org.briarproject.api.lifecycle.Service;
 import org.briarproject.api.lifecycle.ServiceException;
 import org.briarproject.api.plugins.ConnectionManager;
-import org.briarproject.api.plugins.ConnectionRegistry;
 import org.briarproject.api.plugins.Plugin;
 import org.briarproject.api.plugins.PluginCallback;
 import org.briarproject.api.plugins.PluginConfig;
@@ -51,7 +46,7 @@ import javax.inject.Inject;
 import static java.util.logging.Level.INFO;
 import static java.util.logging.Level.WARNING;
 
-class PluginManagerImpl implements PluginManager, Service, EventListener {
+class PluginManagerImpl implements PluginManager, Service {
 
 	private static final Logger LOG =
 			Logger.getLogger(PluginManagerImpl.class.getName());
@@ -59,9 +54,7 @@ class PluginManagerImpl implements PluginManager, Service, EventListener {
 	private final Executor ioExecutor;
 	private final EventBus eventBus;
 	private final PluginConfig pluginConfig;
-	private final Poller poller;
 	private final ConnectionManager connectionManager;
-	private final ConnectionRegistry connectionRegistry;
 	private final SettingsManager settingsManager;
 	private final TransportPropertyManager transportPropertyManager;
 	private final UiCallback uiCallback;
@@ -71,18 +64,14 @@ class PluginManagerImpl implements PluginManager, Service, EventListener {
 
 	@Inject
 	PluginManagerImpl(@IoExecutor Executor ioExecutor, EventBus eventBus,
-			PluginConfig pluginConfig, Poller poller,
-			ConnectionManager connectionManager,
-			ConnectionRegistry connectionRegistry,
+			PluginConfig pluginConfig, ConnectionManager connectionManager,
 			SettingsManager settingsManager,
 			TransportPropertyManager transportPropertyManager,
 			UiCallback uiCallback) {
 		this.ioExecutor = ioExecutor;
 		this.eventBus = eventBus;
 		this.pluginConfig = pluginConfig;
-		this.poller = poller;
 		this.connectionManager = connectionManager;
-		this.connectionRegistry = connectionRegistry;
 		this.settingsManager = settingsManager;
 		this.transportPropertyManager = transportPropertyManager;
 		this.uiCallback = uiCallback;
@@ -93,39 +82,53 @@ class PluginManagerImpl implements PluginManager, Service, EventListener {
 
 	@Override
 	public void startService() throws ServiceException {
+		Collection<SimplexPluginFactory> simplexFactories =
+				pluginConfig.getSimplexFactories();
+		Collection<DuplexPluginFactory> duplexFactories =
+				pluginConfig.getDuplexFactories();
+		int numPlugins = simplexFactories.size() + duplexFactories.size();
+		CountDownLatch latch = new CountDownLatch(numPlugins);
 		// Instantiate and start the simplex plugins
 		LOG.info("Starting simplex plugins");
-		Collection<SimplexPluginFactory> sFactories =
-				pluginConfig.getSimplexFactories();
-		final CountDownLatch sLatch = new CountDownLatch(sFactories.size());
-		for (SimplexPluginFactory factory : sFactories)
-			ioExecutor.execute(new SimplexPluginStarter(factory, sLatch));
+		for (SimplexPluginFactory f : simplexFactories) {
+			TransportId t = f.getId();
+			SimplexPlugin s = f.createPlugin(new SimplexCallback(t));
+			if (s == null) {
+				if (LOG.isLoggable(WARNING))
+					LOG.warning("Could not create plugin for " + t);
+				latch.countDown();
+			} else {
+				plugins.put(t, s);
+				simplexPlugins.add(s);
+				ioExecutor.execute(new PluginStarter(s, latch));
+			}
+		}
 		// Instantiate and start the duplex plugins
 		LOG.info("Starting duplex plugins");
-		Collection<DuplexPluginFactory> dFactories =
-				pluginConfig.getDuplexFactories();
-		final CountDownLatch dLatch = new CountDownLatch(dFactories.size());
-		for (DuplexPluginFactory factory : dFactories)
-			ioExecutor.execute(new DuplexPluginStarter(factory, dLatch));
-		// Wait for the plugins to start
+		for (DuplexPluginFactory f : duplexFactories) {
+			TransportId t = f.getId();
+			DuplexPlugin d = f.createPlugin(new DuplexCallback(t));
+			if (d == null) {
+				if (LOG.isLoggable(WARNING))
+					LOG.warning("Could not create plugin for " + t);
+				latch.countDown();
+			} else {
+				plugins.put(t, d);
+				duplexPlugins.add(d);
+				ioExecutor.execute(new PluginStarter(d, latch));
+			}
+		}
+		// Wait for all the plugins to start
 		try {
-			sLatch.await();
-			dLatch.await();
+			latch.await();
 		} catch (InterruptedException e) {
 			throw new ServiceException(e);
 		}
-		// Listen for events
-		eventBus.addListener(this);
 	}
 
 	@Override
 	public void stopService() throws ServiceException {
-		// Stop listening for events
-		eventBus.removeListener(this);
-		// Stop the poller
-		LOG.info("Stopping poller");
-		poller.stop();
-		final CountDownLatch latch = new CountDownLatch(plugins.size());
+		CountDownLatch latch = new CountDownLatch(plugins.size());
 		// Stop the simplex plugins
 		LOG.info("Stopping simplex plugins");
 		for (SimplexPlugin plugin : simplexPlugins)
@@ -147,6 +150,16 @@ class PluginManagerImpl implements PluginManager, Service, EventListener {
 		return plugins.get(t);
 	}
 
+	@Override
+	public Collection<SimplexPlugin> getSimplexPlugins() {
+		return Collections.unmodifiableList(simplexPlugins);
+	}
+
+	@Override
+	public Collection<DuplexPlugin> getDuplexPlugins() {
+		return Collections.unmodifiableList(duplexPlugins);
+	}
+
 	@Override
 	public Collection<DuplexPlugin> getInvitationPlugins() {
 		List<DuplexPlugin> supported = new ArrayList<DuplexPlugin>();
@@ -163,158 +176,32 @@ class PluginManagerImpl implements PluginManager, Service, EventListener {
 		return Collections.unmodifiableList(supported);
 	}
 
-	@Override
-	public void eventOccurred(Event e) {
-		if (e instanceof ContactStatusChangedEvent) {
-			ContactStatusChangedEvent c = (ContactStatusChangedEvent) e;
-			if (c.isActive()) {
-				// Connect to the newly activated contact
-				connectToContact(c.getContactId());
-			}
-		} else if (e instanceof ConnectionClosedEvent) {
-			ConnectionClosedEvent c = (ConnectionClosedEvent) e;
-			if (!c.isIncoming()) {
-				// Connect to the disconnected contact
-				connectToContact(c.getContactId(), c.getTransportId());
-			}
-		}
-	}
-
-	private void connectToContact(ContactId c) {
-		for (SimplexPlugin s : simplexPlugins)
-			if (s.shouldPoll()) connectToContact(c, s);
-		for (DuplexPlugin d : duplexPlugins)
-			if (d.shouldPoll()) connectToContact(c, d);
-	}
-
-	private void connectToContact(ContactId c, TransportId t) {
-		Plugin p = plugins.get(t);
-		if (p instanceof SimplexPlugin && p.shouldPoll())
-			connectToContact(c, (SimplexPlugin) p);
-		else if (p instanceof DuplexPlugin && p.shouldPoll())
-			connectToContact(c, (DuplexPlugin) p);
-	}
-
-	private void connectToContact(final ContactId c, final SimplexPlugin p) {
-		ioExecutor.execute(new Runnable() {
-			@Override
-			public void run() {
-				TransportId t = p.getId();
-				if (!connectionRegistry.isConnected(c, t)) {
-					TransportConnectionWriter w = p.createWriter(c);
-					if (w != null)
-						connectionManager.manageOutgoingConnection(c, t, w);
-				}
-			}
-		});
-	}
-
-	private void connectToContact(final ContactId c, final DuplexPlugin p) {
-		ioExecutor.execute(new Runnable() {
-			@Override
-			public void run() {
-				TransportId t = p.getId();
-				if (!connectionRegistry.isConnected(c, t)) {
-					DuplexTransportConnection d = p.createConnection(c);
-					if (d != null)
-						connectionManager.manageOutgoingConnection(c, t, d);
-				}
-			}
-		});
-	}
-
-	private class SimplexPluginStarter implements Runnable {
+	private class PluginStarter implements Runnable {
 
-		private final SimplexPluginFactory factory;
-		private final CountDownLatch latch;
-
-		private SimplexPluginStarter(SimplexPluginFactory factory,
-				CountDownLatch latch) {
-			this.factory = factory;
-			this.latch = latch;
-		}
-
-		@Override
-		public void run() {
-			try {
-				TransportId id = factory.getId();
-				SimplexCallback callback = new SimplexCallback(id);
-				SimplexPlugin plugin = factory.createPlugin(callback);
-				if (plugin == null) {
-					if (LOG.isLoggable(INFO)) {
-						String name = factory.getClass().getSimpleName();
-						LOG.info(name + " did not create a plugin");
-					}
-					return;
-				}
-				try {
-					long start = System.currentTimeMillis();
-					boolean started = plugin.start();
-					long duration = System.currentTimeMillis() - start;
-					if (started) {
-						plugins.put(id, plugin);
-						simplexPlugins.add(plugin);
-						if (LOG.isLoggable(INFO)) {
-							String name = plugin.getClass().getSimpleName();
-							LOG.info("Starting " + name + " took " +
-									duration + " ms");
-						}
-					} else {
-						if (LOG.isLoggable(WARNING)) {
-							String name = plugin.getClass().getSimpleName();
-							LOG.warning(name + " did not start");
-						}
-					}
-				} catch (IOException e) {
-					if (LOG.isLoggable(WARNING))
-						LOG.log(WARNING, e.toString(), e);
-				}
-			} finally {
-				latch.countDown();
-			}
-		}
-	}
-
-	private class DuplexPluginStarter implements Runnable {
-
-		private final DuplexPluginFactory factory;
+		private final Plugin plugin;
 		private final CountDownLatch latch;
 
-		private DuplexPluginStarter(DuplexPluginFactory factory,
-				CountDownLatch latch) {
-			this.factory = factory;
+		private PluginStarter(Plugin plugin, CountDownLatch latch) {
+			this.plugin = plugin;
 			this.latch = latch;
 		}
 
 		@Override
 		public void run() {
 			try {
-				TransportId id = factory.getId();
-				DuplexCallback callback = new DuplexCallback(id);
-				DuplexPlugin plugin = factory.createPlugin(callback);
-				if (plugin == null) {
-					if (LOG.isLoggable(INFO)) {
-						String name = factory.getClass().getSimpleName();
-						LOG.info(name + " did not create a plugin");
-					}
-					return;
-				}
 				try {
 					long start = System.currentTimeMillis();
 					boolean started = plugin.start();
 					long duration = System.currentTimeMillis() - start;
 					if (started) {
-						plugins.put(id, plugin);
-						duplexPlugins.add(plugin);
 						if (LOG.isLoggable(INFO)) {
-							String name = plugin.getClass().getSimpleName();
-							LOG.info("Starting " + name + " took " +
-									duration + " ms");
+							LOG.info("Starting plugin " + plugin.getId()
+									+ " took " + duration + " ms");
 						}
 					} else {
 						if (LOG.isLoggable(WARNING)) {
-							String name = plugin.getClass().getSimpleName();
-							LOG.warning(name + " did not start");
+							LOG.warning("Plugin" + plugin.getId()
+									+ " did not start");
 						}
 					}
 				} catch (IOException e) {
@@ -344,8 +231,8 @@ class PluginManagerImpl implements PluginManager, Service, EventListener {
 				plugin.stop();
 				long duration = System.currentTimeMillis() - start;
 				if (LOG.isLoggable(INFO)) {
-					String name = plugin.getClass().getSimpleName();
-					LOG.info("Stopping " + name + " took " + duration + " ms");
+					LOG.info("Stopping plugin " + plugin.getId()
+							+ " took " + duration + " ms");
 				}
 			} catch (IOException e) {
 				if (LOG.isLoggable(WARNING)) LOG.log(WARNING, e.toString(), e);
@@ -431,8 +318,6 @@ class PluginManagerImpl implements PluginManager, Service, EventListener {
 		@Override
 		public void transportEnabled() {
 			eventBus.broadcast(new TransportEnabledEvent(id));
-			Plugin p = plugins.get(id);
-			if (p != null) poller.pollNow(p);
 		}
 
 		@Override
diff --git a/briar-core/src/org/briarproject/plugins/PluginsModule.java b/briar-core/src/org/briarproject/plugins/PluginsModule.java
index 41294866593643833f27c5c9f69fa3218f28fa0d..fc58bc40457002f6d1d9b21fa36d002d360cc96a 100644
--- a/briar-core/src/org/briarproject/plugins/PluginsModule.java
+++ b/briar-core/src/org/briarproject/plugins/PluginsModule.java
@@ -7,14 +7,11 @@ import org.briarproject.api.plugins.BackoffFactory;
 import org.briarproject.api.plugins.ConnectionManager;
 import org.briarproject.api.plugins.ConnectionRegistry;
 import org.briarproject.api.plugins.PluginManager;
-import org.briarproject.api.sync.SyncSessionFactory;
-import org.briarproject.api.system.Timer;
-import org.briarproject.api.transport.KeyManager;
-import org.briarproject.api.transport.StreamReaderFactory;
-import org.briarproject.api.transport.StreamWriterFactory;
+import org.briarproject.api.system.Clock;
 
 import java.security.SecureRandom;
 import java.util.concurrent.Executor;
+import java.util.concurrent.ScheduledExecutorService;
 
 import javax.inject.Inject;
 import javax.inject.Singleton;
@@ -28,6 +25,8 @@ public class PluginsModule {
 	public static class EagerSingletons {
 		@Inject
 		PluginManager pluginManager;
+		@Inject
+		Poller poller;
 	}
 
 	@Provides
@@ -36,33 +35,35 @@ public class PluginsModule {
 	}
 
 	@Provides
+	@Singleton
 	Poller providePoller(@IoExecutor Executor ioExecutor,
-			ConnectionRegistry connectionRegistry, SecureRandom random,
-			Timer timer) {
-		return new PollerImpl(ioExecutor, connectionRegistry, random, timer);
+			ScheduledExecutorService scheduler,
+			ConnectionManager connectionManager,
+			ConnectionRegistry connectionRegistry, PluginManager pluginManager,
+			SecureRandom random, Clock clock, EventBus eventBus) {
+		Poller poller = new Poller(ioExecutor, scheduler, connectionManager,
+				connectionRegistry, pluginManager, random, clock);
+		eventBus.addListener(poller);
+		return poller;
 	}
 
 	@Provides
+	@Singleton
 	ConnectionManager provideConnectionManager(
-			@IoExecutor Executor ioExecutor, KeyManager keyManager,
-			StreamReaderFactory streamReaderFactory,
-			StreamWriterFactory streamWriterFactory,
-			SyncSessionFactory syncSessionFactory,
-			ConnectionRegistry connectionRegistry) {
-		return new ConnectionManagerImpl(ioExecutor, keyManager,
-				streamReaderFactory, streamWriterFactory, syncSessionFactory,
-				connectionRegistry);
+			ConnectionManagerImpl connectionManager) {
+		return connectionManager;
 	}
 
 	@Provides
 	@Singleton
-	ConnectionRegistry provideConnectionRegistry(EventBus eventBus) {
-		return new ConnectionRegistryImpl(eventBus);
+	ConnectionRegistry provideConnectionRegistry(
+			ConnectionRegistryImpl connectionRegistry) {
+		return connectionRegistry;
 	}
 
 	@Provides
 	@Singleton
-	PluginManager getPluginManager(LifecycleManager lifecycleManager,
+	PluginManager providePluginManager(LifecycleManager lifecycleManager,
 			PluginManagerImpl pluginManager) {
 		lifecycleManager.registerService(pluginManager);
 		return pluginManager;
diff --git a/briar-core/src/org/briarproject/plugins/Poller.java b/briar-core/src/org/briarproject/plugins/Poller.java
index 2cacf1561b0bb8a513fd3f91140eeeb9c1367170..20643e43e5c33f2106b056d91ea30b9d6fbdc385 100644
--- a/briar-core/src/org/briarproject/plugins/Poller.java
+++ b/briar-core/src/org/briarproject/plugins/Poller.java
@@ -1,12 +1,203 @@
 package org.briarproject.plugins;
 
+import org.briarproject.api.TransportId;
+import org.briarproject.api.contact.ContactId;
+import org.briarproject.api.event.ConnectionClosedEvent;
+import org.briarproject.api.event.ConnectionOpenedEvent;
+import org.briarproject.api.event.ContactStatusChangedEvent;
+import org.briarproject.api.event.Event;
+import org.briarproject.api.event.EventListener;
+import org.briarproject.api.event.TransportEnabledEvent;
+import org.briarproject.api.lifecycle.IoExecutor;
+import org.briarproject.api.plugins.ConnectionManager;
+import org.briarproject.api.plugins.ConnectionRegistry;
 import org.briarproject.api.plugins.Plugin;
+import org.briarproject.api.plugins.PluginManager;
+import org.briarproject.api.plugins.TransportConnectionWriter;
+import org.briarproject.api.plugins.duplex.DuplexPlugin;
+import org.briarproject.api.plugins.duplex.DuplexTransportConnection;
+import org.briarproject.api.plugins.simplex.SimplexPlugin;
+import org.briarproject.api.system.Clock;
 
-interface Poller {
+import java.security.SecureRandom;
+import java.util.HashMap;
+import java.util.Map;
+import java.util.concurrent.Executor;
+import java.util.concurrent.ScheduledExecutorService;
+import java.util.concurrent.locks.Lock;
+import java.util.concurrent.locks.ReentrantLock;
+import java.util.logging.Logger;
 
-	/** Tells the poller to poll the given plugin immediately. */
-	void pollNow(Plugin p);
+import javax.inject.Inject;
 
-	/** Stops the poller. */
-	void stop();
+import static java.util.concurrent.TimeUnit.MILLISECONDS;
+import static java.util.logging.Level.INFO;
+
+class Poller implements EventListener {
+
+	private static final Logger LOG = Logger.getLogger(Poller.class.getName());
+
+	private final Executor ioExecutor;
+	private final ScheduledExecutorService scheduler;
+	private final ConnectionManager connectionManager;
+	private final ConnectionRegistry connectionRegistry;
+	private final PluginManager pluginManager;
+	private final SecureRandom random;
+	private final Clock clock;
+	private final Lock lock;
+	private final Map<TransportId, PollTask> tasks; // Locking: lock
+
+	@Inject
+	Poller(@IoExecutor Executor ioExecutor, ScheduledExecutorService scheduler,
+			ConnectionManager connectionManager,
+			ConnectionRegistry connectionRegistry, PluginManager pluginManager,
+			SecureRandom random, Clock clock) {
+		this.ioExecutor = ioExecutor;
+		this.scheduler = scheduler;
+		this.connectionManager = connectionManager;
+		this.connectionRegistry = connectionRegistry;
+		this.pluginManager = pluginManager;
+		this.random = random;
+		this.clock = clock;
+		lock = new ReentrantLock();
+		tasks = new HashMap<TransportId, PollTask>();
+	}
+
+	@Override
+	public void eventOccurred(Event e) {
+		if (e instanceof ContactStatusChangedEvent) {
+			ContactStatusChangedEvent c = (ContactStatusChangedEvent) e;
+			if (c.isActive()) {
+				// Connect to the newly activated contact
+				connectToContact(c.getContactId());
+			}
+		} else if (e instanceof ConnectionClosedEvent) {
+			ConnectionClosedEvent c = (ConnectionClosedEvent) e;
+			// Reschedule polling, the polling interval may have decreased
+			reschedule(c.getTransportId());
+			if (!c.isIncoming()) {
+				// Connect to the disconnected contact
+				connectToContact(c.getContactId(), c.getTransportId());
+			}
+		} else if (e instanceof ConnectionOpenedEvent) {
+			ConnectionOpenedEvent c = (ConnectionOpenedEvent) e;
+			// Reschedule polling, the polling interval may have decreased
+			reschedule(c.getTransportId());
+		} else if (e instanceof TransportEnabledEvent) {
+			TransportEnabledEvent t = (TransportEnabledEvent) e;
+			// Poll the newly enabled transport
+			pollNow(t.getTransportId());
+		}
+	}
+
+	private void connectToContact(ContactId c) {
+		for (SimplexPlugin s : pluginManager.getSimplexPlugins())
+			if (s.shouldPoll()) connectToContact(c, s);
+		for (DuplexPlugin d : pluginManager.getDuplexPlugins())
+			if (d.shouldPoll()) connectToContact(c, d);
+	}
+
+	private void connectToContact(ContactId c, TransportId t) {
+		Plugin p = pluginManager.getPlugin(t);
+		if (p instanceof SimplexPlugin && p.shouldPoll())
+			connectToContact(c, (SimplexPlugin) p);
+		else if (p instanceof DuplexPlugin && p.shouldPoll())
+			connectToContact(c, (DuplexPlugin) p);
+	}
+
+	private void connectToContact(final ContactId c, final SimplexPlugin p) {
+		ioExecutor.execute(new Runnable() {
+			@Override
+			public void run() {
+				TransportId t = p.getId();
+				if (!connectionRegistry.isConnected(c, t)) {
+					TransportConnectionWriter w = p.createWriter(c);
+					if (w != null)
+						connectionManager.manageOutgoingConnection(c, t, w);
+				}
+			}
+		});
+	}
+
+	private void connectToContact(final ContactId c, final DuplexPlugin p) {
+		ioExecutor.execute(new Runnable() {
+			@Override
+			public void run() {
+				TransportId t = p.getId();
+				if (!connectionRegistry.isConnected(c, t)) {
+					DuplexTransportConnection d = p.createConnection(c);
+					if (d != null)
+						connectionManager.manageOutgoingConnection(c, t, d);
+				}
+			}
+		});
+	}
+
+	private void reschedule(TransportId t) {
+		Plugin p = pluginManager.getPlugin(t);
+		if (p.shouldPoll()) schedule(p, p.getPollingInterval(), false);
+	}
+
+	private void pollNow(TransportId t) {
+		Plugin p = pluginManager.getPlugin(t);
+		// Randomise next polling interval
+		if (p.shouldPoll()) schedule(p, 0, true);
+	}
+
+	private void schedule(Plugin p, int delay, boolean randomiseNext) {
+		// Replace any later scheduled task for this plugin
+		long due = clock.currentTimeMillis() + delay;
+		TransportId t = p.getId();
+		lock.lock();
+		try {
+			PollTask scheduled = tasks.get(t);
+			if (scheduled == null || due < scheduled.due) {
+				PollTask task = new PollTask(p, due, randomiseNext);
+				tasks.put(t, task);
+				scheduler.schedule(task, delay, MILLISECONDS);
+			}
+		} finally {
+			lock.unlock();
+		}
+	}
+
+	private void poll(final Plugin p) {
+		ioExecutor.execute(new Runnable() {
+			@Override
+			public void run() {
+				TransportId t = p.getId();
+				if (LOG.isLoggable(INFO)) LOG.info("Polling plugin " + t);
+				p.poll(connectionRegistry.getConnectedContacts(t));
+			}
+		});
+	}
+
+	private class PollTask implements Runnable {
+
+		private final Plugin plugin;
+		private final long due;
+		private final boolean randomiseNext;
+
+		private PollTask(Plugin plugin, long due, boolean randomiseNext) {
+			this.plugin = plugin;
+			this.due = due;
+			this.randomiseNext = randomiseNext;
+		}
+
+		@Override
+		public void run() {
+			lock.lock();
+			try {
+				TransportId t = plugin.getId();
+				if (tasks.get(t) != this) return; // Replaced by another task
+				tasks.remove(t);
+			} finally {
+				lock.unlock();
+			}
+			int delay = plugin.getPollingInterval();
+			if (randomiseNext) delay = (int) (delay * random.nextDouble());
+			schedule(plugin, delay, false);
+			poll(plugin);
+		}
+	}
 }
diff --git a/briar-core/src/org/briarproject/plugins/PollerImpl.java b/briar-core/src/org/briarproject/plugins/PollerImpl.java
deleted file mode 100644
index db8e1b0b8d8c85587e0696692e2890372e6c147a..0000000000000000000000000000000000000000
--- a/briar-core/src/org/briarproject/plugins/PollerImpl.java
+++ /dev/null
@@ -1,92 +0,0 @@
-package org.briarproject.plugins;
-
-import org.briarproject.api.TransportId;
-import org.briarproject.api.lifecycle.IoExecutor;
-import org.briarproject.api.plugins.ConnectionRegistry;
-import org.briarproject.api.plugins.Plugin;
-import org.briarproject.api.system.Timer;
-
-import java.security.SecureRandom;
-import java.util.Map;
-import java.util.TimerTask;
-import java.util.concurrent.ConcurrentHashMap;
-import java.util.concurrent.Executor;
-import java.util.logging.Logger;
-
-import javax.inject.Inject;
-
-import static java.util.logging.Level.INFO;
-
-class PollerImpl implements Poller {
-
-	private static final Logger LOG =
-			Logger.getLogger(PollerImpl.class.getName());
-
-	private final Executor ioExecutor;
-	private final ConnectionRegistry connectionRegistry;
-	private final SecureRandom random;
-	private final Timer timer;
-	private final Map<TransportId, PollTask> tasks;
-
-	@Inject
-	PollerImpl(@IoExecutor Executor ioExecutor,
-			ConnectionRegistry connectionRegistry, SecureRandom random,
-			Timer timer) {
-		this.ioExecutor = ioExecutor;
-		this.connectionRegistry = connectionRegistry;
-		this.random = random;
-		this.timer = timer;
-		tasks = new ConcurrentHashMap<TransportId, PollTask>();
-	}
-
-	@Override
-	public void stop() {
-		timer.cancel();
-	}
-
-	@Override
-	public void pollNow(Plugin p) {
-		// Randomise next polling interval
-		if (p.shouldPoll()) schedule(p, 0, true);
-	}
-
-	private void schedule(Plugin p, int interval, boolean randomiseNext) {
-		// Replace any previously scheduled task for this plugin
-		PollTask task = new PollTask(p, randomiseNext);
-		PollTask replaced = tasks.put(p.getId(), task);
-		if (replaced != null) replaced.cancel();
-		timer.schedule(task, interval);
-	}
-
-	private void poll(final Plugin p) {
-		ioExecutor.execute(new Runnable() {
-			@Override
-			public void run() {
-				if (LOG.isLoggable(INFO))
-					LOG.info("Polling " + p.getClass().getSimpleName());
-				p.poll(connectionRegistry.getConnectedContacts(p.getId()));
-			}
-		});
-	}
-
-	private class PollTask extends TimerTask {
-
-		private final Plugin plugin;
-		private final boolean randomiseNext;
-
-		private PollTask(Plugin plugin, boolean randomiseNext) {
-			this.plugin = plugin;
-			this.randomiseNext = randomiseNext;
-		}
-
-		@Override
-		public void run() {
-			tasks.remove(plugin.getId());
-			int interval = plugin.getPollingInterval();
-			if (randomiseNext)
-				interval = (int) (interval * random.nextDouble());
-			schedule(plugin, interval, false);
-			poll(plugin);
-		}
-	}
-}
diff --git a/briar-core/src/org/briarproject/system/SystemModule.java b/briar-core/src/org/briarproject/system/SystemModule.java
index eb2a0d88d28e3bb95608b0bf8794ba74e40d8b2f..8893a86ab08a1ae3c986f8cef0de25752feef299 100644
--- a/briar-core/src/org/briarproject/system/SystemModule.java
+++ b/briar-core/src/org/briarproject/system/SystemModule.java
@@ -1,23 +1,41 @@
 package org.briarproject.system;
 
+import org.briarproject.api.lifecycle.LifecycleManager;
 import org.briarproject.api.system.Clock;
-import org.briarproject.api.system.LocationUtils;
-import org.briarproject.api.system.SeedProvider;
-import org.briarproject.api.system.Timer;
+
+import java.util.concurrent.Executors;
+import java.util.concurrent.ScheduledExecutorService;
+
+import javax.inject.Inject;
+import javax.inject.Singleton;
 
 import dagger.Module;
 import dagger.Provides;
 
 @Module
 public class SystemModule {
+
+	public static class EagerSingletons {
+		@Inject
+		ScheduledExecutorService scheduledExecutorService;
+	}
+
+	private final ScheduledExecutorService scheduler;
+
+	public SystemModule() {
+		scheduler = Executors.newSingleThreadScheduledExecutor();
+	}
+
 	@Provides
 	Clock provideClock() {
 		return new SystemClock();
 	}
 
 	@Provides
-	Timer provideTimer() {
-		return new SystemTimer();
+	@Singleton
+	ScheduledExecutorService provideScheduledExecutorService(
+			LifecycleManager lifecycleManager) {
+		lifecycleManager.registerForShutdown(scheduler);
+		return scheduler;
 	}
-
 }
diff --git a/briar-core/src/org/briarproject/system/SystemTimer.java b/briar-core/src/org/briarproject/system/SystemTimer.java
deleted file mode 100644
index fae6abbc758c7ef4460bcf561d99f18a4bfa2551..0000000000000000000000000000000000000000
--- a/briar-core/src/org/briarproject/system/SystemTimer.java
+++ /dev/null
@@ -1,31 +0,0 @@
-package org.briarproject.system;
-
-import java.util.TimerTask;
-
-import org.briarproject.api.system.Timer;
-
-/** Default timer implementation. */
-public class SystemTimer implements Timer {
-
-	private final java.util.Timer timer = new java.util.Timer(true);
-
-	public void cancel() {
-		timer.cancel();
-	}
-
-	public int purge() {
-		return timer.purge();
-	}
-
-	public void schedule(TimerTask task, long delay) {
-		timer.schedule(task, delay);
-	}
-
-	public void schedule(TimerTask task, long delay, long period) {
-		timer.schedule(task, delay, period);
-	}
-
-	public void scheduleAtFixedRate(TimerTask task, long delay, long period) {
-		timer.scheduleAtFixedRate(task, delay, period);
-	}
-}
diff --git a/briar-core/src/org/briarproject/transport/KeyManagerImpl.java b/briar-core/src/org/briarproject/transport/KeyManagerImpl.java
index 590343ca2cf8506b1d581b864e810db119b7ed65..fdcbe8f30c03506e47beacc44f218adcc095a9db 100644
--- a/briar-core/src/org/briarproject/transport/KeyManagerImpl.java
+++ b/briar-core/src/org/briarproject/transport/KeyManagerImpl.java
@@ -19,7 +19,6 @@ import org.briarproject.api.plugins.PluginConfig;
 import org.briarproject.api.plugins.duplex.DuplexPluginFactory;
 import org.briarproject.api.plugins.simplex.SimplexPluginFactory;
 import org.briarproject.api.system.Clock;
-import org.briarproject.api.system.Timer;
 import org.briarproject.api.transport.KeyManager;
 import org.briarproject.api.transport.StreamContext;
 
@@ -28,6 +27,7 @@ import java.util.Map;
 import java.util.Map.Entry;
 import java.util.concurrent.ConcurrentHashMap;
 import java.util.concurrent.Executor;
+import java.util.concurrent.ScheduledExecutorService;
 import java.util.logging.Logger;
 
 import javax.inject.Inject;
@@ -42,21 +42,22 @@ class KeyManagerImpl implements KeyManager, Service, EventListener {
 	private final DatabaseComponent db;
 	private final CryptoComponent crypto;
 	private final Executor dbExecutor;
+	private final ScheduledExecutorService scheduler;
 	private final PluginConfig pluginConfig;
-	private final Timer timer;
 	private final Clock clock;
 	private final Map<ContactId, Boolean> activeContacts;
 	private final ConcurrentHashMap<TransportId, TransportKeyManager> managers;
 
 	@Inject
 	KeyManagerImpl(DatabaseComponent db, CryptoComponent crypto,
-			@DatabaseExecutor Executor dbExecutor, PluginConfig pluginConfig,
-			Timer timer, Clock clock) {
+			@DatabaseExecutor Executor dbExecutor,
+			ScheduledExecutorService scheduler, PluginConfig pluginConfig,
+			Clock clock) {
 		this.db = db;
 		this.crypto = crypto;
 		this.dbExecutor = dbExecutor;
+		this.scheduler = scheduler;
 		this.pluginConfig = pluginConfig;
-		this.timer = timer;
 		this.clock = clock;
 		// Use a ConcurrentHashMap as a thread-safe set
 		activeContacts = new ConcurrentHashMap<ContactId, Boolean>();
@@ -80,7 +81,8 @@ class KeyManagerImpl implements KeyManager, Service, EventListener {
 					db.addTransport(txn, e.getKey(), e.getValue());
 				for (Entry<TransportId, Integer> e : transports.entrySet()) {
 					TransportKeyManager m = new TransportKeyManager(db, crypto,
-							timer, clock, e.getKey(), e.getValue());
+							dbExecutor, scheduler, clock, e.getKey(),
+							e.getValue());
 					managers.put(e.getKey(), m);
 					m.start(txn);
 				}
@@ -97,12 +99,14 @@ class KeyManagerImpl implements KeyManager, Service, EventListener {
 	public void stopService() {
 	}
 
+	@Override
 	public void addContact(Transaction txn, ContactId c, SecretKey master,
 			long timestamp, boolean alice) throws DbException {
 		for (TransportKeyManager m : managers.values())
 			m.addContact(txn, c, master, timestamp, alice);
 	}
 
+	@Override
 	public StreamContext getStreamContext(ContactId c, TransportId t)
 			throws DbException {
 		// Don't allow outgoing streams to inactive contacts
@@ -123,6 +127,7 @@ class KeyManagerImpl implements KeyManager, Service, EventListener {
 		return ctx;
 	}
 
+	@Override
 	public StreamContext getStreamContext(TransportId t, byte[] tag)
 			throws DbException {
 		TransportKeyManager m = managers.get(t);
@@ -141,6 +146,7 @@ class KeyManagerImpl implements KeyManager, Service, EventListener {
 		return ctx;
 	}
 
+	@Override
 	public void eventOccurred(Event e) {
 		if (e instanceof ContactRemovedEvent) {
 			removeContact(((ContactRemovedEvent) e).getContactId());
@@ -154,6 +160,7 @@ class KeyManagerImpl implements KeyManager, Service, EventListener {
 	private void removeContact(final ContactId c) {
 		activeContacts.remove(c);
 		dbExecutor.execute(new Runnable() {
+			@Override
 			public void run() {
 				for (TransportKeyManager m : managers.values())
 					m.removeContact(c);
diff --git a/briar-core/src/org/briarproject/transport/TransportKeyManager.java b/briar-core/src/org/briarproject/transport/TransportKeyManager.java
index 0c9b95bf4679e5d6bb09f95a8b733b83dc1b605c..1fb37877fc69ff9dca4f21fa1179cfb7c870511f 100644
--- a/briar-core/src/org/briarproject/transport/TransportKeyManager.java
+++ b/briar-core/src/org/briarproject/transport/TransportKeyManager.java
@@ -9,7 +9,6 @@ import org.briarproject.api.db.DatabaseComponent;
 import org.briarproject.api.db.DbException;
 import org.briarproject.api.db.Transaction;
 import org.briarproject.api.system.Clock;
-import org.briarproject.api.system.Timer;
 import org.briarproject.api.transport.StreamContext;
 import org.briarproject.api.transport.TransportKeys;
 import org.briarproject.transport.ReorderingWindow.Change;
@@ -18,10 +17,12 @@ import java.util.HashMap;
 import java.util.Iterator;
 import java.util.Map;
 import java.util.Map.Entry;
-import java.util.TimerTask;
+import java.util.concurrent.Executor;
+import java.util.concurrent.ScheduledExecutorService;
 import java.util.concurrent.locks.ReentrantLock;
 import java.util.logging.Logger;
 
+import static java.util.concurrent.TimeUnit.MILLISECONDS;
 import static java.util.logging.Level.WARNING;
 import static org.briarproject.api.transport.TransportConstants.MAX_CLOCK_DIFFERENCE;
 import static org.briarproject.api.transport.TransportConstants.TAG_LENGTH;
@@ -34,7 +35,8 @@ class TransportKeyManager {
 
 	private final DatabaseComponent db;
 	private final CryptoComponent crypto;
-	private final Timer timer;
+	private final Executor dbExecutor;
+	private final ScheduledExecutorService scheduler;
 	private final Clock clock;
 	private final TransportId transportId;
 	private final long rotationPeriodLength;
@@ -46,11 +48,12 @@ class TransportKeyManager {
 	private final Map<ContactId, MutableTransportKeys> keys;
 
 	TransportKeyManager(DatabaseComponent db, CryptoComponent crypto,
-			Timer timer, Clock clock, TransportId transportId,
-			long maxLatency) {
+			Executor dbExecutor, ScheduledExecutorService scheduler,
+			Clock clock, TransportId transportId, long maxLatency) {
 		this.db = db;
 		this.crypto = crypto;
-		this.timer = timer;
+		this.dbExecutor = dbExecutor;
+		this.scheduler = scheduler;
 		this.clock = clock;
 		this.transportId = transportId;
 		rotationPeriodLength = maxLatency + MAX_CLOCK_DIFFERENCE;
@@ -122,7 +125,19 @@ class TransportKeyManager {
 	}
 
 	private void scheduleKeyRotation(long now) {
-		TimerTask task = new TimerTask() {
+		Runnable task = new Runnable() {
+			@Override
+			public void run() {
+				rotateKeys();
+			}
+		};
+		long delay = rotationPeriodLength - now % rotationPeriodLength;
+		scheduler.schedule(task, delay, MILLISECONDS);
+	}
+
+	private void rotateKeys() {
+		dbExecutor.execute(new Runnable() {
+			@Override
 			public void run() {
 				try {
 					Transaction txn = db.startTransaction(false);
@@ -137,9 +152,7 @@ class TransportKeyManager {
 						LOG.log(WARNING, e.toString(), e);
 				}
 			}
-		};
-		long delay = rotationPeriodLength - now % rotationPeriodLength;
-		timer.schedule(task, delay);
+		});
 	}
 
 	void addContact(Transaction txn, ContactId c, SecretKey master,
diff --git a/briar-core/src/org/briarproject/transport/TransportModule.java b/briar-core/src/org/briarproject/transport/TransportModule.java
index 5cc81e458a00c02b109721c0061bc7ae4c37ac91..4c614ddc69d7505dd2e19879eda73942e96f7fb9 100644
--- a/briar-core/src/org/briarproject/transport/TransportModule.java
+++ b/briar-core/src/org/briarproject/transport/TransportModule.java
@@ -18,7 +18,8 @@ import dagger.Provides;
 public class TransportModule {
 
 	public static class EagerSingletons {
-		@Inject KeyManager keyManager;
+		@Inject
+		KeyManager keyManager;
 	}
 
 	@Provides
@@ -35,7 +36,7 @@ public class TransportModule {
 
 	@Provides
 	@Singleton
-	KeyManager getKeyManager(LifecycleManager lifecycleManager,
+	KeyManager provideKeyManager(LifecycleManager lifecycleManager,
 			EventBus eventBus, KeyManagerImpl keyManager) {
 		lifecycleManager.registerService(keyManager);
 		eventBus.addListener(keyManager);
diff --git a/briar-desktop/src/org/briarproject/system/DesktopSeedProviderModule.java b/briar-desktop/src/org/briarproject/system/DesktopSeedProviderModule.java
new file mode 100644
index 0000000000000000000000000000000000000000..8ed175a8b2c9656a479e2476587c945258def262
--- /dev/null
+++ b/briar-desktop/src/org/briarproject/system/DesktopSeedProviderModule.java
@@ -0,0 +1,19 @@
+package org.briarproject.system;
+
+import org.briarproject.api.system.SeedProvider;
+import org.briarproject.util.OsUtils;
+
+import javax.inject.Singleton;
+
+import dagger.Module;
+import dagger.Provides;
+
+@Module
+public class DesktopSeedProviderModule {
+
+	@Provides
+	@Singleton
+	SeedProvider provideSeedProvider() {
+		return OsUtils.isLinux() ? new LinuxSeedProvider() : null;
+	}
+}
diff --git a/briar-desktop/src/org/briarproject/system/DesktopSystemModule.java b/briar-desktop/src/org/briarproject/system/DesktopSystemModule.java
deleted file mode 100644
index ab811ebba86ee576691dc16c1f1083e006dc8d4c..0000000000000000000000000000000000000000
--- a/briar-desktop/src/org/briarproject/system/DesktopSystemModule.java
+++ /dev/null
@@ -1,31 +0,0 @@
-package org.briarproject.system;
-
-import org.briarproject.api.system.Clock;
-import org.briarproject.api.system.SeedProvider;
-import org.briarproject.api.system.Timer;
-import org.briarproject.util.OsUtils;
-
-import dagger.Module;
-import dagger.Provides;
-
-@Module
-public class DesktopSystemModule {
-
-	@Provides
-	Clock provideClock() {
-		return new SystemClock();
-	}
-
-	@Provides
-	Timer provideTimer() {
-		return new SystemTimer();
-	}
-
-	@Provides
-	SeedProvider provideSeedProvider() {
-		if (OsUtils.isLinux()) {
-			return new LinuxSeedProvider();
-		}
-		return null;
-	}
-}
diff --git a/briar-tests/src/org/briarproject/RunAction.java b/briar-tests/src/org/briarproject/RunAction.java
new file mode 100644
index 0000000000000000000000000000000000000000..ef547ce189deda0120d635bab425dcd693d8334f
--- /dev/null
+++ b/briar-tests/src/org/briarproject/RunAction.java
@@ -0,0 +1,20 @@
+package org.briarproject;
+
+import org.hamcrest.Description;
+import org.jmock.api.Action;
+import org.jmock.api.Invocation;
+
+public class RunAction implements Action {
+
+	@Override
+	public Object invoke(Invocation invocation) throws Throwable {
+		Runnable task = (Runnable) invocation.getParameter(0);
+		task.run();
+		return null;
+	}
+
+	@Override
+	public void describeTo(Description description) {
+		description.appendText("runs a runnable");
+	}
+}
diff --git a/briar-tests/src/org/briarproject/TestSeedProviderModule.java b/briar-tests/src/org/briarproject/TestSeedProviderModule.java
new file mode 100644
index 0000000000000000000000000000000000000000..a239ba7d8dc0954a2446d8c033403f348f549416
--- /dev/null
+++ b/briar-tests/src/org/briarproject/TestSeedProviderModule.java
@@ -0,0 +1,18 @@
+package org.briarproject;
+
+import org.briarproject.api.system.SeedProvider;
+
+import javax.inject.Singleton;
+
+import dagger.Module;
+import dagger.Provides;
+
+@Module
+public class TestSeedProviderModule {
+
+	@Provides
+	@Singleton
+	SeedProvider provideSeedProvider() {
+		return new TestSeedProvider();
+	}
+}
diff --git a/briar-tests/src/org/briarproject/TestSystemModule.java b/briar-tests/src/org/briarproject/TestSystemModule.java
deleted file mode 100644
index 2082896a312abd7e62f3eb25612fd5ee78d98856..0000000000000000000000000000000000000000
--- a/briar-tests/src/org/briarproject/TestSystemModule.java
+++ /dev/null
@@ -1,29 +0,0 @@
-package org.briarproject;
-
-import org.briarproject.api.system.Clock;
-import org.briarproject.api.system.SeedProvider;
-import org.briarproject.api.system.Timer;
-import org.briarproject.system.SystemClock;
-import org.briarproject.system.SystemTimer;
-
-import dagger.Module;
-import dagger.Provides;
-
-@Module
-public class TestSystemModule {
-
-	@Provides
-	Clock provideClock() {
-		return new SystemClock();
-	}
-
-	@Provides
-	Timer provideSystemTimer() {
-		return new SystemTimer();
-	}
-
-	@Provides
-	SeedProvider provideSeedProvider() {
-		return new TestSeedProvider();
-	}
-}
diff --git a/briar-tests/src/org/briarproject/plugins/PluginManagerImplTest.java b/briar-tests/src/org/briarproject/plugins/PluginManagerImplTest.java
index cd2400c7780c73d92a0fc36f7bf7ce45d9973c0f..b8c9e4f6a2e5c470047b3c9686a678943ed66634 100644
--- a/briar-tests/src/org/briarproject/plugins/PluginManagerImplTest.java
+++ b/briar-tests/src/org/briarproject/plugins/PluginManagerImplTest.java
@@ -1,20 +1,13 @@
 package org.briarproject.plugins;
 
 import org.briarproject.BriarTestCase;
-import org.briarproject.ImmediateExecutor;
 import org.briarproject.api.TransportId;
-import org.briarproject.api.contact.ContactId;
-import org.briarproject.api.event.ContactStatusChangedEvent;
 import org.briarproject.api.event.EventBus;
-import org.briarproject.api.event.EventListener;
 import org.briarproject.api.plugins.ConnectionManager;
-import org.briarproject.api.plugins.ConnectionRegistry;
 import org.briarproject.api.plugins.PluginConfig;
-import org.briarproject.api.plugins.TransportConnectionWriter;
 import org.briarproject.api.plugins.duplex.DuplexPlugin;
 import org.briarproject.api.plugins.duplex.DuplexPluginCallback;
 import org.briarproject.api.plugins.duplex.DuplexPluginFactory;
-import org.briarproject.api.plugins.duplex.DuplexTransportConnection;
 import org.briarproject.api.plugins.simplex.SimplexPlugin;
 import org.briarproject.api.plugins.simplex.SimplexPluginCallback;
 import org.briarproject.api.plugins.simplex.SimplexPluginFactory;
@@ -40,11 +33,8 @@ public class PluginManagerImplTest extends BriarTestCase {
 		final Executor ioExecutor = Executors.newSingleThreadExecutor();
 		final EventBus eventBus = context.mock(EventBus.class);
 		final PluginConfig pluginConfig = context.mock(PluginConfig.class);
-		final Poller poller = context.mock(Poller.class);
 		final ConnectionManager connectionManager =
 				context.mock(ConnectionManager.class);
-		final ConnectionRegistry connectionRegistry =
-				context.mock(ConnectionRegistry.class);
 		final SettingsManager settingsManager =
 				context.mock(SettingsManager.class);
 		final TransportPropertyManager transportPropertyManager =
@@ -72,6 +62,12 @@ public class PluginManagerImplTest extends BriarTestCase {
 		final TransportId duplexFailId = new TransportId("duplex1");
 
 		context.checking(new Expectations() {{
+			allowing(simplexPlugin).getId();
+			will(returnValue(simplexId));
+			allowing(simplexFailPlugin).getId();
+			will(returnValue(simplexFailId));
+			allowing(duplexPlugin).getId();
+			will(returnValue(duplexId));
 			// start()
 			// First simplex plugin
 			oneOf(pluginConfig).getSimplexFactories();
@@ -108,21 +104,16 @@ public class PluginManagerImplTest extends BriarTestCase {
 			oneOf(duplexFailFactory).createPlugin(with(any(
 					DuplexPluginCallback.class)));
 			will(returnValue(null)); // Failed to create a plugin
-			// Start listening for events
-			oneOf(eventBus).addListener(with(any(EventListener.class)));
 			// stop()
-			// Stop listening for events
-			oneOf(eventBus).removeListener(with(any(EventListener.class)));
-			// Stop the poller
-			oneOf(poller).stop();
 			// Stop the plugins
 			oneOf(simplexPlugin).stop();
+			oneOf(simplexFailPlugin).stop();
 			oneOf(duplexPlugin).stop();
 		}});
 
 		PluginManagerImpl p = new PluginManagerImpl(ioExecutor, eventBus,
-				pluginConfig, poller, connectionManager, connectionRegistry,
-				settingsManager, transportPropertyManager, uiCallback);
+				pluginConfig, connectionManager, settingsManager,
+				transportPropertyManager, uiCallback);
 
 		// Two plugins should be started and stopped
 		p.startService();
@@ -130,141 +121,4 @@ public class PluginManagerImplTest extends BriarTestCase {
 
 		context.assertIsSatisfied();
 	}
-
-	@Test
-	public void testConnectToNewContact() throws Exception {
-		Mockery context = new Mockery();
-		final Executor ioExecutor = new ImmediateExecutor();
-		final EventBus eventBus = context.mock(EventBus.class);
-		final PluginConfig pluginConfig = context.mock(PluginConfig.class);
-		final Poller poller = context.mock(Poller.class);
-		final ConnectionManager connectionManager =
-				context.mock(ConnectionManager.class);
-		final ConnectionRegistry connectionRegistry =
-				context.mock(ConnectionRegistry.class);
-		final SettingsManager settingsManager =
-				context.mock(SettingsManager.class);
-		final TransportPropertyManager transportPropertyManager =
-				context.mock(TransportPropertyManager.class);
-		final UiCallback uiCallback = context.mock(UiCallback.class);
-		final TransportConnectionWriter transportConnectionWriter =
-				context.mock(TransportConnectionWriter.class);
-		final DuplexTransportConnection duplexTransportConnection =
-				context.mock(DuplexTransportConnection.class);
-
-		final ContactId contactId = new ContactId(234);
-
-		// Two simplex plugins: one supports polling, the other doesn't
-		final SimplexPluginFactory simplexFactory =
-				context.mock(SimplexPluginFactory.class);
-		final SimplexPlugin simplexPlugin = context.mock(SimplexPlugin.class);
-		final TransportId simplexId = new TransportId("simplex");
-		final SimplexPluginFactory simplexFactory1 =
-				context.mock(SimplexPluginFactory.class, "simplexFactory1");
-		final SimplexPlugin simplexPlugin1 =
-				context.mock(SimplexPlugin.class, "simplexPlugin1");
-		final TransportId simplexId1 = new TransportId("simplex1");
-
-		// Two duplex plugins: one supports polling, the other doesn't
-		final DuplexPluginFactory duplexFactory =
-				context.mock(DuplexPluginFactory.class);
-		final DuplexPlugin duplexPlugin = context.mock(DuplexPlugin.class);
-		final TransportId duplexId = new TransportId("duplex");
-		final DuplexPluginFactory duplexFactory1 =
-				context.mock(DuplexPluginFactory.class, "duplexFactory1");
-		final DuplexPlugin duplexPlugin1 =
-				context.mock(DuplexPlugin.class, "duplexPlugin1");
-		final TransportId duplexId1 = new TransportId("duplex1");
-
-		context.checking(new Expectations() {{
-			// start()
-			// First simplex plugin
-			oneOf(pluginConfig).getSimplexFactories();
-			will(returnValue(Arrays.asList(simplexFactory, simplexFactory1)));
-			oneOf(simplexFactory).getId();
-			will(returnValue(simplexId));
-			oneOf(simplexFactory).createPlugin(with(any(
-					SimplexPluginCallback.class)));
-			will(returnValue(simplexPlugin)); // Created
-			oneOf(simplexPlugin).start();
-			will(returnValue(true)); // Started
-			// Second simplex plugin
-			oneOf(simplexFactory1).getId();
-			will(returnValue(simplexId1));
-			oneOf(simplexFactory1).createPlugin(with(any(
-					SimplexPluginCallback.class)));
-			will(returnValue(simplexPlugin1)); // Created
-			oneOf(simplexPlugin1).start();
-			will(returnValue(true)); // Started
-			// First duplex plugin
-			oneOf(pluginConfig).getDuplexFactories();
-			will(returnValue(Arrays.asList(duplexFactory, duplexFactory1)));
-			oneOf(duplexFactory).getId();
-			will(returnValue(duplexId));
-			oneOf(duplexFactory).createPlugin(with(any(
-					DuplexPluginCallback.class)));
-			will(returnValue(duplexPlugin)); // Created
-			oneOf(duplexPlugin).start();
-			will(returnValue(true)); // Started
-			// Second duplex plugin
-			oneOf(duplexFactory1).getId();
-			will(returnValue(duplexId1));
-			oneOf(duplexFactory1).createPlugin(with(any(
-					DuplexPluginCallback.class)));
-			will(returnValue(duplexPlugin1)); // Created
-			oneOf(duplexPlugin1).start();
-			will(returnValue(true)); // Started
-			// Start listening for events
-			oneOf(eventBus).addListener(with(any(EventListener.class)));
-			// eventOccurred()
-			// First simplex plugin
-			oneOf(simplexPlugin).shouldPoll();
-			will(returnValue(true));
-			oneOf(simplexPlugin).getId();
-			will(returnValue(simplexId));
-			oneOf(connectionRegistry).isConnected(contactId, simplexId);
-			will(returnValue(false));
-			oneOf(simplexPlugin).createWriter(contactId);
-			will(returnValue(transportConnectionWriter));
-			oneOf(connectionManager).manageOutgoingConnection(contactId,
-					simplexId, transportConnectionWriter);
-			// Second simplex plugin
-			oneOf(simplexPlugin1).shouldPoll();
-			will(returnValue(false));
-			// First duplex plugin
-			oneOf(duplexPlugin).shouldPoll();
-			will(returnValue(true));
-			oneOf(duplexPlugin).getId();
-			will(returnValue(duplexId));
-			oneOf(connectionRegistry).isConnected(contactId, duplexId);
-			will(returnValue(false));
-			oneOf(duplexPlugin).createConnection(contactId);
-			will(returnValue(duplexTransportConnection));
-			oneOf(connectionManager).manageOutgoingConnection(contactId,
-					duplexId, duplexTransportConnection);
-			// Second duplex plugin
-			oneOf(duplexPlugin1).shouldPoll();
-			will(returnValue(false));
-			// stop()
-			// Stop listening for events
-			oneOf(eventBus).removeListener(with(any(EventListener.class)));
-			// Stop the poller
-			oneOf(poller).stop();
-			// Stop the plugins
-			oneOf(simplexPlugin).stop();
-			oneOf(simplexPlugin1).stop();
-			oneOf(duplexPlugin).stop();
-			oneOf(duplexPlugin1).stop();
-		}});
-
-		PluginManagerImpl p = new PluginManagerImpl(ioExecutor, eventBus,
-				pluginConfig, poller, connectionManager, connectionRegistry,
-				settingsManager, transportPropertyManager, uiCallback);
-
-		p.startService();
-		p.eventOccurred(new ContactStatusChangedEvent(contactId, true));
-		p.stopService();
-
-		context.assertIsSatisfied();
-	}
 }
diff --git a/briar-tests/src/org/briarproject/plugins/PollerTest.java b/briar-tests/src/org/briarproject/plugins/PollerTest.java
new file mode 100644
index 0000000000000000000000000000000000000000..5531177d1b95c7e0f5ef7db678b3c0d295e611b9
--- /dev/null
+++ b/briar-tests/src/org/briarproject/plugins/PollerTest.java
@@ -0,0 +1,354 @@
+package org.briarproject.plugins;
+
+import org.briarproject.BriarTestCase;
+import org.briarproject.ImmediateExecutor;
+import org.briarproject.RunAction;
+import org.briarproject.api.TransportId;
+import org.briarproject.api.contact.ContactId;
+import org.briarproject.api.event.ConnectionClosedEvent;
+import org.briarproject.api.event.ConnectionOpenedEvent;
+import org.briarproject.api.event.ContactStatusChangedEvent;
+import org.briarproject.api.event.TransportEnabledEvent;
+import org.briarproject.api.plugins.ConnectionManager;
+import org.briarproject.api.plugins.ConnectionRegistry;
+import org.briarproject.api.plugins.Plugin;
+import org.briarproject.api.plugins.PluginManager;
+import org.briarproject.api.plugins.TransportConnectionWriter;
+import org.briarproject.api.plugins.duplex.DuplexPlugin;
+import org.briarproject.api.plugins.duplex.DuplexTransportConnection;
+import org.briarproject.api.plugins.simplex.SimplexPlugin;
+import org.briarproject.api.system.Clock;
+import org.jmock.Expectations;
+import org.jmock.Mockery;
+import org.jmock.lib.legacy.ClassImposteriser;
+import org.junit.Test;
+
+import java.security.SecureRandom;
+import java.util.Arrays;
+import java.util.Collections;
+import java.util.List;
+import java.util.concurrent.Executor;
+import java.util.concurrent.ScheduledExecutorService;
+
+import static java.util.concurrent.TimeUnit.MILLISECONDS;
+
+public class PollerTest extends BriarTestCase {
+
+	private final ContactId contactId = new ContactId(234);
+	private final int pollingInterval = 60 * 1000;
+	private final long now = System.currentTimeMillis();
+
+	@Test
+	public void testConnectOnContactStatusChanged() throws Exception {
+		Mockery context = new Mockery();
+		context.setImposteriser(ClassImposteriser.INSTANCE);
+		final Executor ioExecutor = new ImmediateExecutor();
+		final ScheduledExecutorService scheduler =
+				context.mock(ScheduledExecutorService.class);
+		final ConnectionManager connectionManager =
+				context.mock(ConnectionManager.class);
+		final ConnectionRegistry connectionRegistry =
+				context.mock(ConnectionRegistry.class);
+		final PluginManager pluginManager = context.mock(PluginManager.class);
+		final SecureRandom random = context.mock(SecureRandom.class);
+		final Clock clock = context.mock(Clock.class);
+
+		// Two simplex plugins: one supports polling, the other doesn't
+		final SimplexPlugin simplexPlugin = context.mock(SimplexPlugin.class);
+		final SimplexPlugin simplexPlugin1 =
+				context.mock(SimplexPlugin.class, "simplexPlugin1");
+		final TransportId simplexId1 = new TransportId("simplex1");
+		final List<SimplexPlugin> simplexPlugins = Arrays.asList(simplexPlugin,
+				simplexPlugin1);
+		final TransportConnectionWriter simplexWriter =
+				context.mock(TransportConnectionWriter.class);
+
+		// Two duplex plugins: one supports polling, the other doesn't
+		final DuplexPlugin duplexPlugin = context.mock(DuplexPlugin.class);
+		final TransportId duplexId = new TransportId("duplex");
+		final DuplexPlugin duplexPlugin1 =
+				context.mock(DuplexPlugin.class, "duplexPlugin1");
+		final List<DuplexPlugin> duplexPlugins = Arrays.asList(duplexPlugin,
+				duplexPlugin1);
+		final DuplexTransportConnection duplexConnection =
+				context.mock(DuplexTransportConnection.class);
+
+		context.checking(new Expectations() {{
+			// Get the simplex plugins
+			oneOf(pluginManager).getSimplexPlugins();
+			will(returnValue(simplexPlugins));
+			// The first plugin doesn't support polling
+			oneOf(simplexPlugin).shouldPoll();
+			will(returnValue(false));
+			// The second plugin supports polling
+			oneOf(simplexPlugin1).shouldPoll();
+			will(returnValue(true));
+			// Check whether the contact is already connected
+			oneOf(simplexPlugin1).getId();
+			will(returnValue(simplexId1));
+			oneOf(connectionRegistry).isConnected(contactId, simplexId1);
+			will(returnValue(false));
+			// Connect to the contact
+			oneOf(simplexPlugin1).createWriter(contactId);
+			will(returnValue(simplexWriter));
+			// Pass the connection to the connection manager
+			oneOf(connectionManager).manageOutgoingConnection(contactId,
+					simplexId1, simplexWriter);
+			// Get the duplex plugins
+			oneOf(pluginManager).getDuplexPlugins();
+			will(returnValue(duplexPlugins));
+			// The first plugin supports polling
+			oneOf(duplexPlugin).shouldPoll();
+			will(returnValue(true));
+			// Check whether the contact is already connected
+			oneOf(duplexPlugin).getId();
+			will(returnValue(duplexId));
+			oneOf(connectionRegistry).isConnected(contactId, duplexId);
+			will(returnValue(false));
+			// Connect to the contact
+			oneOf(duplexPlugin).createConnection(contactId);
+			will(returnValue(duplexConnection));
+			// Pass the connection to the connection manager
+			oneOf(connectionManager).manageOutgoingConnection(contactId,
+					duplexId, duplexConnection);
+			// The second plugin doesn't support polling
+			oneOf(duplexPlugin1).shouldPoll();
+			will(returnValue(false));
+		}});
+
+		Poller p = new Poller(ioExecutor, scheduler, connectionManager,
+				connectionRegistry, pluginManager, random, clock);
+
+		p.eventOccurred(new ContactStatusChangedEvent(contactId, true));
+
+		context.assertIsSatisfied();
+	}
+
+	@Test
+	public void testRescheduleAndReconnectOnConnectionClosed()
+			throws Exception {
+		Mockery context = new Mockery();
+		context.setImposteriser(ClassImposteriser.INSTANCE);
+		final Executor ioExecutor = new ImmediateExecutor();
+		final ScheduledExecutorService scheduler =
+				context.mock(ScheduledExecutorService.class);
+		final ConnectionManager connectionManager =
+				context.mock(ConnectionManager.class);
+		final ConnectionRegistry connectionRegistry =
+				context.mock(ConnectionRegistry.class);
+		final PluginManager pluginManager = context.mock(PluginManager.class);
+		final SecureRandom random = context.mock(SecureRandom.class);
+		final Clock clock = context.mock(Clock.class);
+
+		final DuplexPlugin plugin = context.mock(DuplexPlugin.class);
+		final TransportId transportId = new TransportId("id");
+		final DuplexTransportConnection duplexConnection =
+				context.mock(DuplexTransportConnection.class);
+
+		context.checking(new Expectations() {{
+			allowing(plugin).getId();
+			will(returnValue(transportId));
+			// reschedule()
+			// Get the plugin
+			oneOf(pluginManager).getPlugin(transportId);
+			will(returnValue(plugin));
+			// The plugin supports polling
+			oneOf(plugin).shouldPoll();
+			will(returnValue(true));
+			// Get the plugin
+			oneOf(pluginManager).getPlugin(transportId);
+			will(returnValue(plugin));
+			// The plugin supports polling
+			oneOf(plugin).shouldPoll();
+			will(returnValue(true));
+			// Schedule the next poll
+			oneOf(plugin).getPollingInterval();
+			will(returnValue(pollingInterval));
+			oneOf(clock).currentTimeMillis();
+			will(returnValue(now));
+			oneOf(scheduler).schedule(with(any(Runnable.class)),
+					with((long) pollingInterval), with(MILLISECONDS));
+			// connectToContact()
+			// Check whether the contact is already connected
+			oneOf(connectionRegistry).isConnected(contactId, transportId);
+			will(returnValue(false));
+			// Connect to the contact
+			oneOf(plugin).createConnection(contactId);
+			will(returnValue(duplexConnection));
+			// Pass the connection to the connection manager
+			oneOf(connectionManager).manageOutgoingConnection(contactId,
+					transportId, duplexConnection);
+		}});
+
+		Poller p = new Poller(ioExecutor, scheduler, connectionManager,
+				connectionRegistry, pluginManager, random, clock);
+
+		p.eventOccurred(new ConnectionClosedEvent(contactId, transportId,
+				false));
+
+		context.assertIsSatisfied();
+	}
+
+
+	@Test
+	public void testRescheduleOnConnectionOpened() throws Exception {
+		Mockery context = new Mockery();
+		context.setImposteriser(ClassImposteriser.INSTANCE);
+		final Executor ioExecutor = new ImmediateExecutor();
+		final ScheduledExecutorService scheduler =
+				context.mock(ScheduledExecutorService.class);
+		final ConnectionManager connectionManager =
+				context.mock(ConnectionManager.class);
+		final ConnectionRegistry connectionRegistry =
+				context.mock(ConnectionRegistry.class);
+		final PluginManager pluginManager = context.mock(PluginManager.class);
+		final SecureRandom random = context.mock(SecureRandom.class);
+		final Clock clock = context.mock(Clock.class);
+
+		final DuplexPlugin plugin = context.mock(DuplexPlugin.class);
+		final TransportId transportId = new TransportId("id");
+
+		context.checking(new Expectations() {{
+			allowing(plugin).getId();
+			will(returnValue(transportId));
+			// Get the plugin
+			oneOf(pluginManager).getPlugin(transportId);
+			will(returnValue(plugin));
+			// The plugin supports polling
+			oneOf(plugin).shouldPoll();
+			will(returnValue(true));
+			// Schedule the next poll
+			oneOf(plugin).getPollingInterval();
+			will(returnValue(pollingInterval));
+			oneOf(clock).currentTimeMillis();
+			will(returnValue(now));
+			oneOf(scheduler).schedule(with(any(Runnable.class)),
+					with((long) pollingInterval), with(MILLISECONDS));
+		}});
+
+		Poller p = new Poller(ioExecutor, scheduler, connectionManager,
+				connectionRegistry, pluginManager, random, clock);
+
+		p.eventOccurred(new ConnectionOpenedEvent(contactId, transportId,
+				false));
+
+		context.assertIsSatisfied();
+	}
+
+	@Test
+	public void testRescheduleDoesNotReplaceEarlierTask() throws Exception {
+		Mockery context = new Mockery();
+		context.setImposteriser(ClassImposteriser.INSTANCE);
+		final Executor ioExecutor = new ImmediateExecutor();
+		final ScheduledExecutorService scheduler =
+				context.mock(ScheduledExecutorService.class);
+		final ConnectionManager connectionManager =
+				context.mock(ConnectionManager.class);
+		final ConnectionRegistry connectionRegistry =
+				context.mock(ConnectionRegistry.class);
+		final PluginManager pluginManager = context.mock(PluginManager.class);
+		final SecureRandom random = context.mock(SecureRandom.class);
+		final Clock clock = context.mock(Clock.class);
+
+		final DuplexPlugin plugin = context.mock(DuplexPlugin.class);
+		final TransportId transportId = new TransportId("id");
+
+		context.checking(new Expectations() {{
+			allowing(plugin).getId();
+			will(returnValue(transportId));
+			// First event
+			// Get the plugin
+			oneOf(pluginManager).getPlugin(transportId);
+			will(returnValue(plugin));
+			// The plugin supports polling
+			oneOf(plugin).shouldPoll();
+			will(returnValue(true));
+			// Schedule the next poll
+			oneOf(plugin).getPollingInterval();
+			will(returnValue(pollingInterval));
+			oneOf(clock).currentTimeMillis();
+			will(returnValue(now));
+			oneOf(scheduler).schedule(with(any(Runnable.class)),
+					with((long) pollingInterval), with(MILLISECONDS));
+			// Second event
+			// Get the plugin
+			oneOf(pluginManager).getPlugin(transportId);
+			will(returnValue(plugin));
+			// The plugin supports polling
+			oneOf(plugin).shouldPoll();
+			will(returnValue(true));
+			// Don't replace the previously scheduled task, due earlier
+			oneOf(plugin).getPollingInterval();
+			will(returnValue(pollingInterval));
+			oneOf(clock).currentTimeMillis();
+			will(returnValue(now + 1));
+		}});
+
+		Poller p = new Poller(ioExecutor, scheduler, connectionManager,
+				connectionRegistry, pluginManager, random, clock);
+
+		p.eventOccurred(new ConnectionOpenedEvent(contactId, transportId,
+				false));
+		p.eventOccurred(new ConnectionOpenedEvent(contactId, transportId,
+				false));
+
+		context.assertIsSatisfied();
+	}
+
+	@Test
+	public void testPollOnTransportEnabled() throws Exception {
+		Mockery context = new Mockery();
+		context.setImposteriser(ClassImposteriser.INSTANCE);
+		final Executor ioExecutor = new ImmediateExecutor();
+		final ScheduledExecutorService scheduler =
+				context.mock(ScheduledExecutorService.class);
+		final ConnectionManager connectionManager =
+				context.mock(ConnectionManager.class);
+		final ConnectionRegistry connectionRegistry =
+				context.mock(ConnectionRegistry.class);
+		final PluginManager pluginManager = context.mock(PluginManager.class);
+		final SecureRandom random = context.mock(SecureRandom.class);
+		final Clock clock = context.mock(Clock.class);
+
+		final Plugin plugin = context.mock(Plugin.class);
+		final TransportId transportId = new TransportId("id");
+		final List<ContactId> connected = Collections.singletonList(contactId);
+
+		context.checking(new Expectations() {{
+			allowing(plugin).getId();
+			will(returnValue(transportId));
+			// Get the plugin
+			oneOf(pluginManager).getPlugin(transportId);
+			will(returnValue(plugin));
+			// The plugin supports polling
+			oneOf(plugin).shouldPoll();
+			will(returnValue(true));
+			// Schedule a polling task immediately
+			oneOf(clock).currentTimeMillis();
+			will(returnValue(now));
+			oneOf(scheduler).schedule(with(any(Runnable.class)), with(0L),
+					with(MILLISECONDS));
+			will(new RunAction());
+			// Running the polling task schedules the next polling task
+			oneOf(plugin).getPollingInterval();
+			will(returnValue(pollingInterval));
+			oneOf(random).nextDouble();
+			will(returnValue(0.5));
+			oneOf(clock).currentTimeMillis();
+			will(returnValue(now));
+			oneOf(scheduler).schedule(with(any(Runnable.class)),
+					with((long) (pollingInterval * 0.5)), with(MILLISECONDS));
+			// Poll the plugin
+			oneOf(connectionRegistry).getConnectedContacts(transportId);
+			will(returnValue(connected));
+			oneOf(plugin).poll(connected);
+		}});
+
+		Poller p = new Poller(ioExecutor, scheduler, connectionManager,
+				connectionRegistry, pluginManager, random, clock);
+
+		p.eventOccurred(new TransportEnabledEvent(transportId));
+
+		context.assertIsSatisfied();
+	}
+}
diff --git a/briar-tests/src/org/briarproject/transport/TransportKeyManagerTest.java b/briar-tests/src/org/briarproject/transport/TransportKeyManagerTest.java
index 68df77e4138205d0c6368371b8637998b2779fac..4a32490c953999232fb0190c05affd658bb85b2e 100644
--- a/briar-tests/src/org/briarproject/transport/TransportKeyManagerTest.java
+++ b/briar-tests/src/org/briarproject/transport/TransportKeyManagerTest.java
@@ -1,6 +1,7 @@
 package org.briarproject.transport;
 
 import org.briarproject.BriarTestCase;
+import org.briarproject.RunAction;
 import org.briarproject.TestUtils;
 import org.briarproject.api.TransportId;
 import org.briarproject.api.contact.ContactId;
@@ -9,7 +10,6 @@ import org.briarproject.api.crypto.SecretKey;
 import org.briarproject.api.db.DatabaseComponent;
 import org.briarproject.api.db.Transaction;
 import org.briarproject.api.system.Clock;
-import org.briarproject.api.system.Timer;
 import org.briarproject.api.transport.IncomingKeys;
 import org.briarproject.api.transport.OutgoingKeys;
 import org.briarproject.api.transport.StreamContext;
@@ -28,8 +28,10 @@ import java.util.LinkedHashMap;
 import java.util.List;
 import java.util.Map;
 import java.util.Random;
-import java.util.TimerTask;
+import java.util.concurrent.Executor;
+import java.util.concurrent.ScheduledExecutorService;
 
+import static java.util.concurrent.TimeUnit.MILLISECONDS;
 import static org.briarproject.api.transport.TransportConstants.MAX_CLOCK_DIFFERENCE;
 import static org.briarproject.api.transport.TransportConstants.REORDERING_WINDOW_SIZE;
 import static org.briarproject.api.transport.TransportConstants.TAG_LENGTH;
@@ -56,11 +58,12 @@ public class TransportKeyManagerTest extends BriarTestCase {
 		Mockery context = new Mockery();
 		final DatabaseComponent db = context.mock(DatabaseComponent.class);
 		final CryptoComponent crypto = context.mock(CryptoComponent.class);
-		final Timer timer = context.mock(Timer.class);
+		final Executor dbExecutor = context.mock(Executor.class);
+		final ScheduledExecutorService scheduler =
+				context.mock(ScheduledExecutorService.class);
 		final Clock clock = context.mock(Clock.class);
 
-		final Map<ContactId, TransportKeys> loaded =
-				new LinkedHashMap<ContactId, TransportKeys>();
+		final Map<ContactId, TransportKeys> loaded = new LinkedHashMap<>();
 		final TransportKeys shouldRotate = createTransportKeys(900, 0);
 		final TransportKeys shouldNotRotate = createTransportKeys(1000, 0);
 		loaded.put(contactId, shouldRotate);
@@ -90,12 +93,12 @@ public class TransportKeyManagerTest extends BriarTestCase {
 			oneOf(db).updateTransportKeys(txn,
 					Collections.singletonMap(contactId, rotated));
 			// Schedule key rotation at the start of the next rotation period
-			oneOf(timer).schedule(with(any(TimerTask.class)),
-					with(rotationPeriodLength - 1));
+			oneOf(scheduler).schedule(with(any(Runnable.class)),
+					with(rotationPeriodLength - 1), with(MILLISECONDS));
 		}});
 
 		TransportKeyManager transportKeyManager = new TransportKeyManager(db,
-				crypto, timer, clock, transportId, maxLatency);
+				crypto, dbExecutor, scheduler, clock, transportId, maxLatency);
 		transportKeyManager.start(txn);
 
 		context.assertIsSatisfied();
@@ -106,7 +109,9 @@ public class TransportKeyManagerTest extends BriarTestCase {
 		Mockery context = new Mockery();
 		final DatabaseComponent db = context.mock(DatabaseComponent.class);
 		final CryptoComponent crypto = context.mock(CryptoComponent.class);
-		final Timer timer = context.mock(Timer.class);
+		final Executor dbExecutor = context.mock(Executor.class);
+		final ScheduledExecutorService scheduler =
+				context.mock(ScheduledExecutorService.class);
 		final Clock clock = context.mock(Clock.class);
 
 		final boolean alice = true;
@@ -135,7 +140,7 @@ public class TransportKeyManagerTest extends BriarTestCase {
 		}});
 
 		TransportKeyManager transportKeyManager = new TransportKeyManager(db,
-				crypto, timer, clock, transportId, maxLatency);
+				crypto, dbExecutor, scheduler, clock, transportId, maxLatency);
 		// The timestamp is 1 ms before the start of rotation period 1000
 		long timestamp = rotationPeriodLength * 1000 - 1;
 		transportKeyManager.addContact(txn, contactId, masterKey, timestamp,
@@ -150,13 +155,15 @@ public class TransportKeyManagerTest extends BriarTestCase {
 		Mockery context = new Mockery();
 		final DatabaseComponent db = context.mock(DatabaseComponent.class);
 		final CryptoComponent crypto = context.mock(CryptoComponent.class);
-		final Timer timer = context.mock(Timer.class);
+		final Executor dbExecutor = context.mock(Executor.class);
+		final ScheduledExecutorService scheduler =
+				context.mock(ScheduledExecutorService.class);
 		final Clock clock = context.mock(Clock.class);
 
 		final Transaction txn = new Transaction(null, false);
 
 		TransportKeyManager transportKeyManager = new TransportKeyManager(db,
-				crypto, timer, clock, transportId, maxLatency);
+				crypto, dbExecutor, scheduler, clock, transportId, maxLatency);
 		assertNull(transportKeyManager.getStreamContext(txn, contactId));
 
 		context.assertIsSatisfied();
@@ -168,7 +175,9 @@ public class TransportKeyManagerTest extends BriarTestCase {
 		Mockery context = new Mockery();
 		final DatabaseComponent db = context.mock(DatabaseComponent.class);
 		final CryptoComponent crypto = context.mock(CryptoComponent.class);
-		final Timer timer = context.mock(Timer.class);
+		final Executor dbExecutor = context.mock(Executor.class);
+		final ScheduledExecutorService scheduler =
+				context.mock(ScheduledExecutorService.class);
 		final Clock clock = context.mock(Clock.class);
 
 		final boolean alice = true;
@@ -198,7 +207,7 @@ public class TransportKeyManagerTest extends BriarTestCase {
 		}});
 
 		TransportKeyManager transportKeyManager = new TransportKeyManager(db,
-				crypto, timer, clock, transportId, maxLatency);
+				crypto, dbExecutor, scheduler, clock, transportId, maxLatency);
 		// The timestamp is at the start of rotation period 1000
 		long timestamp = rotationPeriodLength * 1000;
 		transportKeyManager.addContact(txn, contactId, masterKey, timestamp,
@@ -213,7 +222,9 @@ public class TransportKeyManagerTest extends BriarTestCase {
 		Mockery context = new Mockery();
 		final DatabaseComponent db = context.mock(DatabaseComponent.class);
 		final CryptoComponent crypto = context.mock(CryptoComponent.class);
-		final Timer timer = context.mock(Timer.class);
+		final Executor dbExecutor = context.mock(Executor.class);
+		final ScheduledExecutorService scheduler =
+				context.mock(ScheduledExecutorService.class);
 		final Clock clock = context.mock(Clock.class);
 
 		final boolean alice = true;
@@ -245,7 +256,7 @@ public class TransportKeyManagerTest extends BriarTestCase {
 		}});
 
 		TransportKeyManager transportKeyManager = new TransportKeyManager(db,
-				crypto, timer, clock, transportId, maxLatency);
+				crypto, dbExecutor, scheduler, clock, transportId, maxLatency);
 		// The timestamp is at the start of rotation period 1000
 		long timestamp = rotationPeriodLength * 1000;
 		transportKeyManager.addContact(txn, contactId, masterKey, timestamp,
@@ -271,7 +282,9 @@ public class TransportKeyManagerTest extends BriarTestCase {
 		Mockery context = new Mockery();
 		final DatabaseComponent db = context.mock(DatabaseComponent.class);
 		final CryptoComponent crypto = context.mock(CryptoComponent.class);
-		final Timer timer = context.mock(Timer.class);
+		final Executor dbExecutor = context.mock(Executor.class);
+		final ScheduledExecutorService scheduler =
+				context.mock(ScheduledExecutorService.class);
 		final Clock clock = context.mock(Clock.class);
 
 		final boolean alice = true;
@@ -299,7 +312,7 @@ public class TransportKeyManagerTest extends BriarTestCase {
 		}});
 
 		TransportKeyManager transportKeyManager = new TransportKeyManager(db,
-				crypto, timer, clock, transportId, maxLatency);
+				crypto, dbExecutor, scheduler, clock, transportId, maxLatency);
 		// The timestamp is at the start of rotation period 1000
 		long timestamp = rotationPeriodLength * 1000;
 		transportKeyManager.addContact(txn, contactId, masterKey, timestamp,
@@ -315,13 +328,15 @@ public class TransportKeyManagerTest extends BriarTestCase {
 		Mockery context = new Mockery();
 		final DatabaseComponent db = context.mock(DatabaseComponent.class);
 		final CryptoComponent crypto = context.mock(CryptoComponent.class);
-		final Timer timer = context.mock(Timer.class);
+		final Executor dbExecutor = context.mock(Executor.class);
+		final ScheduledExecutorService scheduler =
+				context.mock(ScheduledExecutorService.class);
 		final Clock clock = context.mock(Clock.class);
 
 		final boolean alice = true;
 		final TransportKeys transportKeys = createTransportKeys(1000, 0);
 		// Keep a copy of the tags
-		final List<byte[]> tags = new ArrayList<byte[]>();
+		final List<byte[]> tags = new ArrayList<>();
 		final Transaction txn = new Transaction(null, false);
 
 		context.checking(new Expectations() {{
@@ -352,7 +367,7 @@ public class TransportKeyManagerTest extends BriarTestCase {
 		}});
 
 		TransportKeyManager transportKeyManager = new TransportKeyManager(db,
-				crypto, timer, clock, transportId, maxLatency);
+				crypto, dbExecutor, scheduler, clock, transportId, maxLatency);
 		// The timestamp is at the start of rotation period 1000
 		long timestamp = rotationPeriodLength * 1000;
 		transportKeyManager.addContact(txn, contactId, masterKey, timestamp,
@@ -381,7 +396,9 @@ public class TransportKeyManagerTest extends BriarTestCase {
 		Mockery context = new Mockery();
 		final DatabaseComponent db = context.mock(DatabaseComponent.class);
 		final CryptoComponent crypto = context.mock(CryptoComponent.class);
-		final Timer timer = context.mock(Timer.class);
+		final Executor dbExecutor = context.mock(Executor.class);
+		final ScheduledExecutorService scheduler =
+				context.mock(ScheduledExecutorService.class);
 		final Clock clock = context.mock(Clock.class);
 
 		final TransportKeys transportKeys = createTransportKeys(1000, 0);
@@ -408,9 +425,11 @@ public class TransportKeyManagerTest extends BriarTestCase {
 				will(new EncodeTagAction());
 			}
 			// Schedule key rotation at the start of the next rotation period
-			oneOf(timer).schedule(with(any(TimerTask.class)),
-					with(rotationPeriodLength));
-			will(new RunTimerTaskAction());
+			oneOf(scheduler).schedule(with(any(Runnable.class)),
+					with(rotationPeriodLength), with(MILLISECONDS));
+			will(new RunAction());
+			oneOf(dbExecutor).execute(with(any(Runnable.class)));
+			will(new RunAction());
 			// Start a transaction for key rotation
 			oneOf(db).startTransaction(false);
 			will(returnValue(txn1));
@@ -431,14 +450,14 @@ public class TransportKeyManagerTest extends BriarTestCase {
 			oneOf(db).updateTransportKeys(txn1,
 					Collections.singletonMap(contactId, rotated));
 			// Schedule key rotation at the start of the next rotation period
-			oneOf(timer).schedule(with(any(TimerTask.class)),
-					with(rotationPeriodLength));
+			oneOf(scheduler).schedule(with(any(Runnable.class)),
+					with(rotationPeriodLength), with(MILLISECONDS));
 			// Commit the key rotation transaction
 			oneOf(db).endTransaction(txn1);
 		}});
 
 		TransportKeyManager transportKeyManager = new TransportKeyManager(db,
-				crypto, timer, clock, transportId, maxLatency);
+				crypto, dbExecutor, scheduler, clock, transportId, maxLatency);
 		transportKeyManager.start(txn);
 		assertTrue(txn1.isComplete());
 
@@ -483,19 +502,4 @@ public class TransportKeyManagerTest extends BriarTestCase {
 			description.appendText("encodes a tag");
 		}
 	}
-
-	private static class RunTimerTaskAction implements Action {
-
-		@Override
-		public Object invoke(Invocation invocation) throws Throwable {
-			TimerTask task = (TimerTask) invocation.getParameter(0);
-			task.run();
-			return null;
-		}
-
-		@Override
-		public void describeTo(Description description) {
-			description.appendText("schedules a timer task");
-		}
-	}
 }
diff --git a/briar-api/src/org/briarproject/api/plugins/PluginManager.java b/briar-api/src/org/briarproject/api/plugins/PluginManager.java
index 838c66ff86e1ac74a41c507f9d48b6774038c247..18e71c2ec2e75ae4790c7e1580b7b95e75b1700c 100644
--- a/briar-api/src/org/briarproject/api/plugins/PluginManager.java
+++ b/briar-api/src/org/briarproject/api/plugins/PluginManager.java
@@ -2,24 +2,39 @@ package org.briarproject.api.plugins;
 
 import org.briarproject.api.TransportId;
 import org.briarproject.api.plugins.duplex.DuplexPlugin;
+import org.briarproject.api.plugins.simplex.SimplexPlugin;
 
 import java.util.Collection;
 
 /**
- * Responsible for starting transport plugins at startup, stopping them at
- * shutdown, and providing access to plugins for exchanging invitations.
+ * Responsible for starting transport plugins at startup and stopping them at
+ * shutdown.
  */
 public interface PluginManager {
 
 	/**
 	 * Returns the plugin for the given transport, or null if no such plugin
-	 * is running.
+	 * has been created.
 	 */
 	Plugin getPlugin(TransportId t);
 
-	/** Returns any running duplex plugins that support invitations. */
+	/**
+	 * Returns any simplex plugins that have been created.
+	 */
+	Collection<SimplexPlugin> getSimplexPlugins();
+
+	/**
+	 * Returns any duplex plugins that have been created.
+	 */
+	Collection<DuplexPlugin> getDuplexPlugins();
+
+	/**
+	 * Returns any duplex plugins that support invitations.
+	 */
 	Collection<DuplexPlugin> getInvitationPlugins();
 
-	/** Returns any running duplex plugins that support key agreement. */
+	/**
+	 * Returns any duplex plugins that support key agreement.
+	 */
 	Collection<DuplexPlugin> getKeyAgreementPlugins();
 }